/*
 * (C) Copyright 2004-2007
 * Cavium Networks
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <asm/au1x00.h>
#include <asm/mipsregs.h>
#include "octeon_boot.h"
#include <pci.h>
#include <miiphy.h>
#include <lib_octeon_shared.h>
#include <lib_octeon.h>


#if defined(CONFIG_PCI)
extern void init_octeon_pci (void);

void pci_init_board (void)
{
	init_octeon_pci();
}
#endif

/* Boot bus init for flash and peripheral access */
#define FLASH_RoundUP(_Dividend, _Divisor) (((_Dividend)+(_Divisor))/(_Divisor))
flash_info_t flash_info[CFG_MAX_FLASH_BANKS];	/* info for FLASH chips */
int octeon_boot_bus_init_board(void)
{
    cvmx_mio_boot_reg_cfgx_t reg_cfg;
    cvmx_mio_boot_reg_timx_t __attribute__((unused)) reg_tim;
    uint64_t ECLK_MHZ, ECLK_PERIOD;

    /* Read CPU clock multiplier */
    uint64_t data = cvmx_read_csr((uint64_t)0x80011F0000008510ull);
    data = data >> 18;
    data &= 0x1f;
    data *= 50; /* cpu_ref = 50 default */
    data += 200;   /* Give us some margin  */
    ECLK_MHZ = data;
    ECLK_PERIOD = 1000000 / ECLK_MHZ; /* eclk period (psecs) */

    /*
    ** Set the TWSI clock to a conservative 100KHz.  Compute the
    ** clocks M divider based on the core clock.
    **
    **  TWSI freq = (core freq) / (20 x (M+1) x (thp+1) x 2^N)
    **	M = ((core freq) / (20 x (TWSI freq) x (thp+1) x 2^N)) - 1
    */

#ifndef TWSI_BUS_FREQ
#define TWSI_BUS_FREQ 	(100000) /* 100KHz */
#endif
#ifndef TWSI_THP
#define TWSI_THP	    (24) /* TCLK half period (default 24) */
#endif

    int M_divider, N_divider;
    for (N_divider = 0; N_divider < 8; ++N_divider) {
        M_divider = ((ECLK_MHZ * 1000000) / (20 * TWSI_BUS_FREQ * (TWSI_THP+1) * (1<<N_divider))) - 1;
        if (M_divider < 16) break;
    }

    cvmx_mio_tws_sw_twsi_t sw_twsi;
    sw_twsi.u64 = 0;
    sw_twsi.s.v = 1;
    sw_twsi.s.op = 0x6;
    sw_twsi.s.eop_ia = 0x3;
    sw_twsi.s.d = ((M_divider&0xf) << 3) | ((N_divider&0x7) << 0);
    cvmx_write_csr(CVMX_MIO_TWS_SW_TWSI, sw_twsi.u64);


#ifdef CFG_FLASH_SIZE
    /* Remap flash part so that it is all addressable on boot bus, with alias
    ** at 0x1fc00000 so that the data mapped at the default location (0x1fc00000) is
    ** still available at the same address */
    reg_cfg.u64 = 0;
    reg_cfg.s.en = 1;
    reg_cfg.s.size = ((CFG_FLASH_SIZE + 0x400000) >> 16) - 1;  /* In 64k blocks, + 4MByte alias of low 4Mbytes of flash */
    reg_cfg.s.base = ((CFG_FLASH_BASE >> 16) & 0x1fff);  /* Mask to put physical address in boot bus window */
    cvmx_write_csr(CVMX_MIO_BOOT_REG_CFG0, reg_cfg.u64);
#endif

    /* Set up regions for CompactFlash */
    /* Attribute memory region */
#ifdef OCTEON_CF_ATTRIB_CHIP_SEL
    reg_cfg.u64 = 0;
    reg_cfg.s.en = 1;
#ifdef OCTEON_CF_16_BIT_BUS
    reg_cfg.s.width = 1;
#endif
    reg_cfg.s.base = ((OCTEON_CF_ATTRIB_BASE_ADDR >> 16) & 0x1fff);  /* Mask to put physical address in boot bus window */
    cvmx_write_csr(CVMX_MIO_BOOT_REG_CFGX(OCTEON_CF_ATTRIB_CHIP_SEL), reg_cfg.u64);

    reg_tim.u64 = 0;
    reg_tim.s.wait = 0x3f;
    reg_tim.s.page = 0x3f;
    reg_tim.s.wr_hld = FLASH_RoundUP((FLASH_RoundUP(70000ULL, ECLK_PERIOD) - 2), 4);
    reg_tim.s.rd_hld = FLASH_RoundUP((FLASH_RoundUP(100000ULL, ECLK_PERIOD) - 1), 4);
    reg_tim.s.we = FLASH_RoundUP((FLASH_RoundUP(150000ULL, ECLK_PERIOD) - 1), 4);
    reg_tim.s.oe = FLASH_RoundUP((FLASH_RoundUP(270000ULL, ECLK_PERIOD) - 1), 4);
    reg_tim.s.ce = FLASH_RoundUP((FLASH_RoundUP(30000ULL, ECLK_PERIOD) - 2), 4);
    cvmx_write_csr(CVMX_MIO_BOOT_REG_TIMX(OCTEON_CF_ATTRIB_CHIP_SEL), reg_tim.u64);
#endif

#ifdef OCTEON_CF_COMMON_CHIP_SEL
    /* Common memory region */
    reg_cfg.u64 = 0;
    reg_cfg.s.en = 1;
#ifdef OCTEON_CF_16_BIT_BUS
    reg_cfg.s.width = 1;
#endif
    reg_cfg.s.base = ((OCTEON_CF_COMMON_BASE_ADDR >> 16) & 0x1fff);  /* Mask to put physical address in boot bus window */
    cvmx_write_csr(CVMX_MIO_BOOT_REG_CFGX(OCTEON_CF_COMMON_CHIP_SEL), reg_cfg.u64);

    reg_tim.u64 = 0;
    reg_tim.s.wait = (FLASH_RoundUP(30000ULL, ECLK_PERIOD) - 1);
    reg_tim.s.waitm = 1;
    reg_tim.s.page = 0x3f;
    reg_tim.s.wr_hld = FLASH_RoundUP((FLASH_RoundUP(30000ULL, ECLK_PERIOD) - 1), 4);
    reg_tim.s.rd_hld = FLASH_RoundUP((FLASH_RoundUP(100000ULL, ECLK_PERIOD) - 1), 4);
    reg_tim.s.we = FLASH_RoundUP((FLASH_RoundUP(150000ULL, ECLK_PERIOD) - 1), 4);
    reg_tim.s.oe = FLASH_RoundUP((FLASH_RoundUP(125000ULL, ECLK_PERIOD) - 1), 4);
    reg_tim.s.ce = FLASH_RoundUP((FLASH_RoundUP(30000ULL, ECLK_PERIOD) - 2), 4);
    cvmx_write_csr(CVMX_MIO_BOOT_REG_TIMX(OCTEON_CF_COMMON_CHIP_SEL), reg_tim.u64);
#endif


#ifdef OCTEON_CF_TRUE_IDE_CS0_CHIP_SEL
    reg_cfg.u64 = 0;
    reg_cfg.s.en = 1;
    reg_cfg.s.width = 1;
    reg_cfg.s.base = ((OCTEON_CF_TRUE_IDE_CS0_ADDR >> 16) & 0x1fff);  /* Mask to put physical address in boot bus window */
    cvmx_write_csr(CVMX_MIO_BOOT_REG_CFGX(OCTEON_CF_TRUE_IDE_CS0_CHIP_SEL), reg_cfg.u64);

    reg_tim.u64 = 0;
    reg_tim.s.wait = (FLASH_RoundUP(300000ULL, ECLK_PERIOD) - 1);
    reg_tim.s.page = 0x3f;
    reg_tim.s.wr_hld = FLASH_RoundUP((FLASH_RoundUP(300000ULL, ECLK_PERIOD) - 1), 4);
    reg_tim.s.rd_hld = FLASH_RoundUP((FLASH_RoundUP(1000000ULL, ECLK_PERIOD) - 1), 4);
    reg_tim.s.we = FLASH_RoundUP((FLASH_RoundUP(1500000ULL, ECLK_PERIOD) - 1), 4);
    reg_tim.s.oe = FLASH_RoundUP((FLASH_RoundUP(1250000ULL, ECLK_PERIOD) - 1), 4);
    reg_tim.s.ce = FLASH_RoundUP((FLASH_RoundUP(300000ULL, ECLK_PERIOD) - 2), 4);
    cvmx_write_csr(CVMX_MIO_BOOT_REG_TIMX(OCTEON_CF_TRUE_IDE_CS0_CHIP_SEL), reg_tim.u64);
#endif


#ifdef OCTEON_CF_TRUE_IDE_CS1_CHIP_SEL
    reg_cfg.u64 = 0;
    reg_cfg.s.en = 1;
    reg_cfg.s.width = 1;
    reg_cfg.s.base = ((OCTEON_CF_TRUE_IDE_CS1_ADDR >> 16) & 0x1fff);  /* Mask to put physical address in boot bus window */
    cvmx_write_csr(CVMX_MIO_BOOT_REG_CFGX(OCTEON_CF_TRUE_IDE_CS1_CHIP_SEL), reg_cfg.u64);

    reg_tim.u64 = 0;
    reg_tim.s.wait = (FLASH_RoundUP(30000ULL, ECLK_PERIOD) - 1);
    reg_tim.s.page = 0x3f;
    reg_tim.s.wr_hld = FLASH_RoundUP((FLASH_RoundUP(30000ULL, ECLK_PERIOD) - 1), 4);
    reg_tim.s.rd_hld = FLASH_RoundUP((FLASH_RoundUP(100000ULL, ECLK_PERIOD) - 1), 4);
    reg_tim.s.we = FLASH_RoundUP((FLASH_RoundUP(150000ULL, ECLK_PERIOD) - 1), 4);
    reg_tim.s.oe = FLASH_RoundUP((FLASH_RoundUP(125000ULL, ECLK_PERIOD) - 1), 4);
    reg_tim.s.ce = FLASH_RoundUP((FLASH_RoundUP(30000ULL, ECLK_PERIOD) - 2), 4);
    cvmx_write_csr(CVMX_MIO_BOOT_REG_TIMX(OCTEON_CF_TRUE_IDE_CS1_CHIP_SEL), reg_tim.u64);
#endif


#ifdef OCTEON_CHAR_LED_CHIP_SEL
    /* Setup region for 4 char LED display */
    reg_cfg.u64 = 0;
    reg_cfg.s.en = 1;
    reg_cfg.s.base = ((OCTEON_CHAR_LED_BASE_ADDR >> 16) & 0x1fff);  /* Mask to put physical address in boot bus window */
    cvmx_write_csr(CVMX_MIO_BOOT_REG_CFGX(4), reg_cfg.u64);
#endif

#ifdef OCTEON_PAL_CHIP_SEL
    /* Setup region for PAL access */
    reg_cfg.u64 = 0;
    reg_cfg.s.en = 1;
    reg_cfg.s.base = ((OCTEON_PAL_BASE_ADDR >> 16) & 0x1fff);  /* Mask to put physical address in boot bus window */
    cvmx_write_csr(CVMX_MIO_BOOT_REG_CFGX(OCTEON_PAL_CHIP_SEL), reg_cfg.u64);
#endif

#if CONFIG_RAM_RESIDENT
    /* Install the 2nd moveable region to the debug exception main entry
        point. This way the debugger will work even if there isn't any
        flash. */
    extern void debugHandler_entrypoint(void);
    const uint64_t *handler_code = (const uint64_t *)debugHandler_entrypoint;
    int count;
    cvmx_write_csr(CVMX_MIO_BOOT_LOC_CFGX(1), (1<<31) | (0x1fc00480>>7<<3));
    cvmx_write_csr(CVMX_MIO_BOOT_LOC_ADR, 0x80);
    for (count=0; count<128/8; count++)
        cvmx_write_csr(CVMX_MIO_BOOT_LOC_DAT, *handler_code++);
#endif

    return(0);
}



int checkboard (void)
{
    DECLARE_GLOBAL_DATA_PTR;

#if defined(CONFIG_MII) || (CONFIG_COMMANDS & CFG_CMD_MII)
    /* Enable MDIO */
    cvmx_write_csr(CVMX_SMI_EN, 1);

    /* Configure the PHYS for the proper LED mode */
    {
        int i;
        for (i = 2; i <= 5; i++)
        {
            miiphy_write(i, 0x16, 3);   /* set register page */
            /* Write LED modes */
            miiphy_write(i, 0x10, 0x5777);
            miiphy_write(i, 0x13, 0x0070);
            miiphy_write(i, 0x16, 0);   /* set register page */
        }
    }
#endif
    if (octeon_show_info())
    {

        int pal_rev_maj = 0;
        int pal_rev_min = 0;
        int voltage_100ths = 0;
        int voltage_1s = 0;
        int mcu_rev_maj = 0;
        int mcu_rev_min = 0;

        if (octeon_read64_byte(OCTEON_PAL_BASE_ADDR)==0xa5 && octeon_read64_byte(OCTEON_PAL_BASE_ADDR+1)==0x5a)
        {
            pal_rev_maj = octeon_read64_byte(OCTEON_PAL_BASE_ADDR+2);
            pal_rev_min = octeon_read64_byte(OCTEON_PAL_BASE_ADDR+3);
            if ((octeon_read64_byte(OCTEON_PAL_BASE_ADDR+4))>0xf)
            {
                voltage_1s = 0;
                voltage_100ths = (600+(31-octeon_read64_byte(OCTEON_PAL_BASE_ADDR+4))*25);
            }
            else
            {
                voltage_1s = 1;
                voltage_100ths = ((15-octeon_read64_byte(OCTEON_PAL_BASE_ADDR+4))*5);
            }
        }

        if (twsii_mcu_read(0x00)==0xa5 && twsii_mcu_read(0x01)==0x5a)
        {
            gd->mcu_rev_maj = mcu_rev_maj = twsii_mcu_read(2);
            gd->mcu_rev_min = mcu_rev_min = twsii_mcu_read(3);
        }

        printf("PAL rev: %d.%02d, MCU rev: %d.%02d, CPU voltage: %d.%02d\n",
               pal_rev_maj, pal_rev_min, mcu_rev_maj, mcu_rev_min, voltage_1s, voltage_100ths);

    }

    if (octeon_show_info())
    {
        /* Display CPU speed on display */
        if ((octeon_read64_byte(OCTEON_PAL_BASE_ADDR+4))>0xf)
        {
            char tmp[10];
            sprintf(tmp,"%d 0.%.2d ",gd->cpu_clock_mhz,(600+(31-octeon_read64_byte(OCTEON_PAL_BASE_ADDR+4))*25)/10);
            octeon_led_str_write(tmp);
        }
        else
        {
            char tmp[10];
            sprintf(tmp,"%d 1.%.2d ",gd->cpu_clock_mhz,(15-octeon_read64_byte(OCTEON_PAL_BASE_ADDR+4))*5);
            octeon_led_str_write(tmp);
        }
    }
    else
    {
        octeon_led_str_write("Boot    ");
    }

    return 0;
}



int early_board_init(void)
{
    int cpu_ref = DEFAULT_CPU_REF_FREQUENCY_MHZ;

    DECLARE_GLOBAL_DATA_PTR;

    memset((void *)&(gd->mac_desc), 0x0, sizeof(octeon_eeprom_mac_addr_t));
    memset((void *)&(gd->clock_desc), 0x0, sizeof(octeon_eeprom_clock_desc_t));
    memset((void *)&(gd->board_desc), 0x0, sizeof(octeon_eeprom_board_desc_t));

    /* NOTE: this is early in the init process, so the serial port is not yet configured */

    /* Populate global data from eeprom */
    uint8_t ee_buf[OCTEON_EEPROM_MAX_TUPLE_LENGTH];
    int addr;

    /* Determine board type/rev */
    strncpy((char *)(gd->board_desc.serial_str), "unknown", SERIAL_LEN);
    addr = octeon_tlv_get_tuple_addr(CFG_DEF_EEPROM_ADDR, EEPROM_BOARD_DESC_TYPE, 0, ee_buf, OCTEON_EEPROM_MAX_TUPLE_LENGTH);
    if (addr >= 0)
    {
        memcpy((void *)&(gd->board_desc), ee_buf, sizeof(octeon_eeprom_board_desc_t));
    }
    else
    {
        gd->flags |= GD_FLG_BOARD_DESC_MISSING;
        gd->board_desc.rev_minor = 0;
        gd->board_desc.board_type = CVMX_BOARD_TYPE_EBH5200;
        gd->board_desc.rev_major = 1;
    }

    /* Even though the CPU ref freq is stored in the clock descriptor, we don't read it here.  The MCU
    ** reads it and configures the clock, and we read how the clock is actually configured.
    ** The bootloader does not need to read the clock descriptor tuple for normal operation on
    ** ebh5200 boards
    */
    cpu_ref = octeon_mcu_read_cpu_ref();

    addr = octeon_tlv_get_tuple_addr(CFG_DEF_EEPROM_ADDR, EEPROM_CLOCK_DESC_TYPE, 0, ee_buf, OCTEON_EEPROM_MAX_TUPLE_LENGTH);
    if (addr >= 0)
    {
        memcpy((void *)&(gd->clock_desc), ee_buf, sizeof(octeon_eeprom_clock_desc_t));
        gd->ddr_clock_mhz = gd->clock_desc.ddr_clock_mhz;
    }
    else
    {
        gd->ddr_clock_mhz = EBH5200_DEF_DRAM_FREQ;
    }

    /* CN52XX has a fixed internally generated 50 MHz reference clock */
    gd->ddr_ref_hertz = cpu_ref * 1000000;


    /* Some sanity checks */
    if (cpu_ref <= 0)
    {
        /* Default if cpu reference clock reading fails. */
        cpu_ref = DEFAULT_CPU_REF_FREQUENCY_MHZ;
    }
    if (gd->ddr_clock_mhz < 100 || gd->ddr_clock_mhz > 2000)
    {
        gd->ddr_clock_mhz = EBH5200_DEF_DRAM_FREQ;
    }


    addr = octeon_tlv_get_tuple_addr(CFG_DEF_EEPROM_ADDR, EEPROM_MAC_ADDR_TYPE, 0, ee_buf, OCTEON_EEPROM_MAX_TUPLE_LENGTH);
    if (addr >= 0)
    {
        memcpy((void *)&(gd->mac_desc), ee_buf, sizeof(octeon_eeprom_mac_addr_t));
    }
    else
    {
        /* Make up some MAC addresses */
        gd->mac_desc.count = 255;
        gd->mac_desc.mac_addr_base[0] = 0x00;
        gd->mac_desc.mac_addr_base[1] = 0xDE;
        gd->mac_desc.mac_addr_base[2] = 0xAD;
        gd->mac_desc.mac_addr_base[3] = (gd->board_desc.rev_major<<4) | gd->board_desc.rev_minor;
        gd->mac_desc.mac_addr_base[4] = gd->board_desc.serial_str[0];
        gd->mac_desc.mac_addr_base[5] = 0x00;
    }


    /* Read CPU clock multiplier */
    uint64_t data = cvmx_read_csr((uint64_t)0x80011F0000008510ull);
    data = data >> 18;
    data &= 0x1f;
    gd->cpu_clock_mhz = data * cpu_ref;

    /* adjust for 33.33 Mhz clock */
    if (cpu_ref == 33)
        gd->cpu_clock_mhz += (data)/4 + data/8;

    if (gd->cpu_clock_mhz < 100 || gd->cpu_clock_mhz > 1100)
    {
        gd->cpu_clock_mhz = DEFAULT_ECLK_FREQ_MHZ;
    }

    octeon_led_str_write("Booting.");
    return 0;

}
void octeon_led_str_write(const char *str)
{
    octeon_led_str_write_std(str);
}
