/*
 * (C) Copyright 2004,2005
 * Cavium Networks
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <asm/au1x00.h>
#include <asm/mipsregs.h>
#include "octeon_boot.h"
#include <pci.h>
#include <lib_octeon_shared.h>
#include <lib_octeon.h>
#include <miiphy.h>




typedef union {
  uint16_t  u16;
  struct {
    uint16_t
            busy  : 1,
            rsv   : 2,
            mode  : 1,
            op    : 2,
            device: 5,
            reg   : 5;
  } s;
} smi_cmd;

typedef union {
  uint16_t  u16;
  struct {
    uint16_t
            busy  : 1,
            op    : 3,
            mode  : 2,
            rsv   : 4,
            ptr   : 6;
  } s;
} marvel_status_cmd;

#if (CONFIG_COMMANDS & CFG_CMD_MII)
#define mdio_write(x,y,z) miiphy_write(x,y,z)
#define mdio_read(x,y,z) miiphy_read(x,y,z)

/* MDIO functions to set BCM53118 */
static int BCM53118_MDIO_SET( unsigned short page, unsigned short reg_addr, uint16_t R24, uint16_t R25, uint16_t R26, uint16_t R27)
{
	unsigned long data;
	uint16_t value,result;

	// Step 1 :
	value = 0x0;
	value = value | page;
	value = value <<8;
	value = value | 0x1;
	mdio_write(0x1e, 0x10, value);

	//Step 2:
	mdio_write(0x1e, 0x18, R24);
	mdio_write(0x1e, 0x19, R25);
	mdio_write(0x1e, 0x1a, R26);
	mdio_write(0x1e, 0x1b, R27);
	
	//Step 3:
	value = 0x0;
	value = value | reg_addr;
	value = value <<8;
	value = value | 0x1;
	mdio_write(0x1e, 0x11, value);

	//Step3:
	do
	{
		mdio_read(0x1e, 0x11, &result);
        } while (result & 0x3);

	return 0;

}


static int BCM53118_MDIO_READ( unsigned short page, unsigned short reg_addr, uint16_t *R24, uint16_t *R25, uint16_t *R26, uint16_t *R27)
{
	unsigned long data;
	uint16_t value,result;

	// Step 1 :
	value = 0x0;
	value = value | page;
	value = value <<8;
	value = value | 0x1;
	mdio_write(0x1e, 0x10, value);

	//Step 2:
	value = 0x0;
	value = value | reg_addr;
	value = value <<8;
	value = value | 0x2;
	mdio_write(0x1e, 0x11, value);

	//Step3:
	do
	{
		mdio_read(0x1e, 0x11, &result);
        } while (result & 0x3);

	//Step4:
	mdio_read(0x1e, 0x18, R24);
	mdio_read(0x1e, 0x19, R25);
	mdio_read(0x1e, 0x1a, R26);
	mdio_read(0x1e, 0x1b, R27);

	return 0;

}

static int BCM53118_ini(void)
{

	/*Broadcom header control disable */
 BCM53118_MDIO_SET(0x2,0x3,0x0,0x0,0x0,0x0);  
	
   	/* Force IMP speed/mode */
 BCM53118_MDIO_SET(0x0,0x0e,0x8b,0x0,0x0,0x0);  

	        /* Set up port based VLAN */
		BCM53118_MDIO_SET(0x31,0x0,0x10f,0x0,0x0,0x0); /* Port 0 */ 
		BCM53118_MDIO_SET(0x31,0x2,0x10f,0x0,0x0,0x0);  /* Port 1 */
		BCM53118_MDIO_SET(0x31,0x4,0x10f,0x0,0x0,0x0);   /* Port 2 */
		BCM53118_MDIO_SET(0x31,0x6,0x10f,0x0,0x0,0x0);  /* Port 3 */
		BCM53118_MDIO_SET(0x31,0x8,0x110,0x0,0x0,0x0); /* Port 4 */
		BCM53118_MDIO_SET(0x31,0xa,0x120,0x0,0x0,0x0);/* Port 5 */
		BCM53118_MDIO_SET(0x31,0xc,0x140,0x0,0x0,0x0);/* Port 6 */
		BCM53118_MDIO_SET(0x31,0xe,0x180,0x0,0x0,0x0);/* Port 7 */
		BCM53118_MDIO_SET(0x31,0x10,0x1ff,0x0,0x0,0x0);   /* IMP */




	/* Must disable STP for all ports, as
    	** disabling forwarding via HW_FWDG_EN pin changes
    	** the defaults */

	BCM53118_MDIO_SET(0x0,0x0,0x0,0x0,0x0,0x0);
	BCM53118_MDIO_SET(0x0,0x1,0x0,0x0,0x0,0x0);
	BCM53118_MDIO_SET(0x0,0x2,0x0,0x0,0x0,0x0);
	BCM53118_MDIO_SET(0x0,0x3,0x0,0x0,0x0,0x0);
	BCM53118_MDIO_SET(0x0,0x4,0x0,0x0,0x0,0x0);
	BCM53118_MDIO_SET(0x0,0x5,0x0,0x0,0x0,0x0);
	BCM53118_MDIO_SET(0x0,0x6,0x0,0x0,0x0,0x0);
	BCM53118_MDIO_SET(0x0,0x7,0x0,0x0,0x0,0x0);

	/* Strange setting from broadcom to set single IMP mode */
	BCM53118_MDIO_SET(0x2,0x0,0x80,0x0,0x0,0x0);

	 /* Now enable forwarding */
	BCM53118_MDIO_SET(0x0,0x0b,0x7,0x0,0x0,0x0);  	

	BCM53118_MDIO_SET(0x2,0x50,0x0d,0x0,0x0,0x0);  
	BCM53118_MDIO_SET(0x0,0x08,0x1c,0x0,0x0,0x0);	


	/* Errata sheet */
       /* initial to correct value to identify the ADC function. */
	miiphy_write(0x0,0x17,0x0f76);
	miiphy_write(0x1,0x17,0x0f76);
	miiphy_write(0x2,0x17,0x0f76);
	miiphy_write(0x3,0x17,0x0f76);
	miiphy_write(0x4,0x17,0x0f76);
	miiphy_write(0x5,0x17,0x0f76);
	miiphy_write(0x6,0x17,0x0f76);
	miiphy_write(0x7,0x17,0x0f76);

	miiphy_write(0x0,0x15,0x0040);
	miiphy_write(0x1,0x15,0x0040);
	miiphy_write(0x2,0x15,0x0040);
	miiphy_write(0x3,0x15,0x0040);
	miiphy_write(0x4,0x15,0x0040);
	miiphy_write(0x5,0x15,0x0040);
	miiphy_write(0x6,0x15,0x0040);
	miiphy_write(0x7,0x15,0x0040);


	/* Device may take a longer time to link up with specific marvell phy product. */
	miiphy_write(0x0,0x17,0x0f75);
	miiphy_write(0x1,0x17,0x0f75);
	miiphy_write(0x2,0x17,0x0f75);
	miiphy_write(0x3,0x17,0x0f75);
	miiphy_write(0x4,0x17,0x0f75);
	miiphy_write(0x5,0x17,0x0f75);
	miiphy_write(0x6,0x17,0x0f75);
	miiphy_write(0x7,0x17,0x0f75);

	miiphy_write(0x0,0x15,0x003d);
	miiphy_write(0x1,0x15,0x003d);
	miiphy_write(0x2,0x15,0x003d);
	miiphy_write(0x3,0x15,0x003d);
	miiphy_write(0x4,0x15,0x003d);
	miiphy_write(0x5,0x15,0x003d);
	miiphy_write(0x6,0x15,0x003d);
	miiphy_write(0x7,0x15,0x003d);

	/* Device may take a longer time to link up with specific marvell phy product. */
	miiphy_write(0x0,0x17,0x0f08);
	miiphy_write(0x1,0x17,0x0f08);
	miiphy_write(0x2,0x17,0x0f08);
	miiphy_write(0x3,0x17,0x0f08);
	miiphy_write(0x4,0x17,0x0f08);
	miiphy_write(0x5,0x17,0x0f08);
	miiphy_write(0x6,0x17,0x0f08);
	miiphy_write(0x7,0x17,0x0f08);

	miiphy_write(0x0,0x15,0x0001);
	miiphy_write(0x1,0x15,0x0001);
	miiphy_write(0x2,0x15,0x0001);
	miiphy_write(0x3,0x15,0x0001);
	miiphy_write(0x4,0x15,0x0001);
	miiphy_write(0x5,0x15,0x0001);
	miiphy_write(0x6,0x15,0x0001);
	miiphy_write(0x7,0x15,0x0001);

	miiphy_write(0x0,0x17,0x0f00);
	miiphy_write(0x1,0x17,0x0f00);
	miiphy_write(0x2,0x17,0x0f00);
	miiphy_write(0x3,0x17,0x0f00);
	miiphy_write(0x4,0x17,0x0f00);
	miiphy_write(0x5,0x17,0x0f00);
	miiphy_write(0x6,0x17,0x0f00);
	miiphy_write(0x7,0x17,0x0f00);

	return 0;
}



#endif /* MII */

#if defined(CONFIG_PCI)
extern void init_octeon_pci (void);

void pci_init_board (void)
{
	init_octeon_pci();
}
#endif


int octeon_get_board_major_rev(void)
{
    DECLARE_GLOBAL_DATA_PTR;
    return(gd->board_desc.rev_major);
}

int octeon_get_board_minor_rev(void)
{
    DECLARE_GLOBAL_DATA_PTR;
    return(gd->board_desc.rev_minor);
}

int checkboard (void)
{
    DECLARE_GLOBAL_DATA_PTR;
    if (octeon_show_info())
    {

        int pal_rev_maj = 0;
        int pal_rev_min = 0;
        int voltage_100ths = 0;
        int voltage_1s = 0;
        int mcu_rev_maj = 0;
        int mcu_rev_min = 0;

        if (octeon_read64_byte(OCTEON_PAL_BASE_ADDR)==0xa5 && octeon_read64_byte(OCTEON_PAL_BASE_ADDR+1)==0x5a)
        {
            pal_rev_maj = octeon_read64_byte(OCTEON_PAL_BASE_ADDR+2);
            pal_rev_min = octeon_read64_byte(OCTEON_PAL_BASE_ADDR+3);
            if ((octeon_read64_byte(OCTEON_PAL_BASE_ADDR+4))>0xf)
            {
                voltage_1s = 0;
                voltage_100ths = (600+(31-octeon_read64_byte(OCTEON_PAL_BASE_ADDR+4))*25);
            }
            else
            {
                voltage_1s = 1;
                voltage_100ths = ((15-octeon_read64_byte(OCTEON_PAL_BASE_ADDR+4))*5);
            }
        }

        if (twsii_mcu_read(0x00)==0xa5 && twsii_mcu_read(0x01)==0x5a)
        {
            gd->mcu_rev_maj = mcu_rev_maj = twsii_mcu_read(2);
            gd->mcu_rev_min = mcu_rev_min = twsii_mcu_read(3);
        }

        printf("PAL rev: %d.%02d, MCU rev: %d.%02d, CPU voltage: %d.%02d\n",
               pal_rev_maj, pal_rev_min, mcu_rev_maj, mcu_rev_min, voltage_1s, voltage_100ths);

    }

    if (octeon_show_info())
    {
        /* Display CPU speed on display */

        if ((octeon_read64_byte(OCTEON_PAL_BASE_ADDR+4))>0xf)
        {
            char tmp[10];
            sprintf(tmp,"%d 0.%.2d ",gd->cpu_clock_mhz,(600+(31-octeon_read64_byte(OCTEON_PAL_BASE_ADDR+4))*25)/10);
            octeon_led_str_write(tmp);
        }
        else
        {
            char tmp[10];
            sprintf(tmp,"%d 1.%.2d ",gd->cpu_clock_mhz,(15-octeon_read64_byte(OCTEON_PAL_BASE_ADDR+4))*5);
            octeon_led_str_write(tmp);
        }
    }
    else
    {
        octeon_led_str_write("Boot    ");
    }


    /* Set up switch complex */   


    /* Force us into GMII mode, enable interface */
    /* NOTE:  Applications expect this to be set appropriately
    ** by the bootloader, and will configure the interface accordingly */
    cvmx_write_csr (CVMX_GMXX_INF_MODE (0), 0x2); // change from 0x3 to 0x2 -- by Joe

    /* Enable SMI to talk with the GMII switch */
    cvmx_write_csr (CVMX_SMI_EN, 0x1);
    cvmx_read_csr(CVMX_SMI_EN);


    cvmx_write_csr(CVMX_ASXX_RX_CLK_SETX(0, 0), 0x0);
    cvmx_write_csr(CVMX_ASXX_TX_CLK_SETX(0, 0), 0x19);

//    cvmx_write_csr (0x0001180008000010, 0xf); // force port 0 to 1000 fullduplex mode
//    cvmx_write_csr (0x0001180008000810, 0xf); // force port 1 to 1000 fullduplex mode
//    cvmx_write_csr (0x0001180008001010, 0xf); // force port 2 to 1000 fullduplex mode

    /*Init and urn on test led  */
	octeon_gpio_cfg_output(2);
	octeon_gpio_cfg_output(3);
	octeon_gpio_cfg_output(4);
	octeon_gpio_cfg_output(9);
	octeon_gpio_cfg_output(10);		
	octeon_gpio_cfg_output(11);	
	octeon_gpio_cfg_output(13);
	octeon_gpio_cfg_output(14);	
	octeon_gpio_cfg_output(15);
	octeon_gpio_cfg_output(16);
			
  octeon_gpio_clr(13); // turn on test LED
  octeon_gpio_set(16); //turn off DMZ LED

#if (CONFIG_COMMANDS & CFG_CMD_MII)
	// reset switch 53118
	octeon_gpio_cfg_output(5);
	octeon_gpio_clr(5);
	udelay(60000); /*Wait 60ms*/
	octeon_gpio_set(5);
	udelay(20000); /*Wait 20ms*/
	/* BCM53118 initial  */ 
	BCM53118_ini();
#endif  /* MII */

    return 0;
}



int early_board_init(void)
{
    int cpu_ref = 50; // modified back to 50 -- by roger

    DECLARE_GLOBAL_DATA_PTR;

    memset((void *)&(gd->mac_desc), 0x0, sizeof(octeon_eeprom_mac_addr_t));
    memset((void *)&(gd->clock_desc), 0x0, sizeof(octeon_eeprom_clock_desc_t));
    memset((void *)&(gd->board_desc), 0x0, sizeof(octeon_eeprom_board_desc_t));

    /* NOTE: this is early in the init process, so the serial port is not yet configured */


    /* Populate global data from eeprom */
    uint8_t ee_buf[OCTEON_EEPROM_MAX_TUPLE_LENGTH];
    int addr;

    addr = octeon_tlv_get_tuple_addr(CFG_DEF_EEPROM_ADDR, EEPROM_CLOCK_DESC_TYPE, 0, ee_buf, OCTEON_EEPROM_MAX_TUPLE_LENGTH);
    if (addr >= 0)
    {
        memcpy((void *)&(gd->clock_desc), ee_buf, sizeof(octeon_eeprom_clock_desc_t));
    }


    /* Determine board type/rev */
    octeon_gpio_cfg_input(1);

    strncpy((char *)(gd->board_desc.serial_str), "unknown", SERIAL_LEN);
    addr = octeon_tlv_get_tuple_addr(CFG_DEF_EEPROM_ADDR, EEPROM_BOARD_DESC_TYPE, 0, ee_buf, OCTEON_EEPROM_MAX_TUPLE_LENGTH);
    if (addr >= 0)
    {
        memcpy((void *)&(gd->board_desc), ee_buf, sizeof(octeon_eeprom_board_desc_t));
    }
    else
    {
        gd->flags |= GD_FLG_BOARD_DESC_MISSING;
        gd->board_desc.rev_minor = 0;
	  gd->board_desc.board_type = CVMX_BOARD_TYPE_CUST_SRX5308;	
        gd->board_desc.rev_major = 2;
    }


    /* Even though the CPU ref freq is stored in the clock descriptor, we don't read it here.  The MCU
    ** reads it and configures the clock, and we read how the clock is actually configured.
    ** The bootloader does not need to read the clock descriptor tuple for normal operation on
    ** ebh3100 boards
    */
//    cpu_ref = octeon_mcu_read_cpu_ref();
//    gd->ddr_clock_mhz = octeon_mcu_read_ddr_clock();
//Joe change CPU frq = 33 x 9
//Joe change DDR frq = 266 x 2
      cpu_ref = 50;
      gd->ddr_clock_mhz = 200;    

    /* CN50XX has a fixed internally generated 50 MHz reference clock */
    if (OCTEON_IS_MODEL(OCTEON_CN50XX)) {
        gd->ddr_ref_hertz = 50000000;
        gd->ddr_clock_mhz = gd->clock_desc.ddr_clock_mhz;
    }


    /* Some sanity checks */
    if (cpu_ref <= 0)
    {
        /* Default to 33 -- by Joe */
        cpu_ref = 50;
    }
    if (gd->ddr_clock_mhz <= 0)
    {
        /* Default to 200 */
        gd->ddr_clock_mhz = 200;
    }

    addr = octeon_tlv_get_tuple_addr(CFG_DEF_EEPROM_ADDR, EEPROM_MAC_ADDR_TYPE, 0, ee_buf, OCTEON_EEPROM_MAX_TUPLE_LENGTH);
    if (addr >= 0)
    {
        memcpy((void *)&(gd->mac_desc), ee_buf, sizeof(octeon_eeprom_mac_addr_t));
    }
    else
    {
        /* Make up some MAC addresses */
        gd->mac_desc.count = 255;
        gd->mac_desc.mac_addr_base[0] = 0x00;
        gd->mac_desc.mac_addr_base[1] = 0x1E;
        gd->mac_desc.mac_addr_base[2] = 0x2A;
        gd->mac_desc.mac_addr_base[3] = 0x3D;
        gd->mac_desc.mac_addr_base[4] = 0x28;
        gd->mac_desc.mac_addr_base[5] = 0x4A;
    }



    /* Read CPU clock multiplier */
    cvmx_dbg_data_t dbg_data;
    dbg_data.u64 = cvmx_read_csr(CVMX_DBG_DATA);
    int mul = dbg_data.cn30xx.c_mul;
    gd->cpu_clock_mhz = mul * cpu_ref;

    /* adjust for 33.33 Mhz clock */
    if (cpu_ref == 33)
        gd->cpu_clock_mhz += (mul)/4 + mul/8;

    if (gd->cpu_clock_mhz < 100 || gd->cpu_clock_mhz > 900)
    {
        gd->cpu_clock_mhz = DEFAULT_ECLK_FREQ_MHZ;
    }
    return 0;

}
void octeon_led_str_write(const char *str)
{
    octeon_led_str_write_std(str);
}

