#include <common.h>
#include <config.h>
#include <watchdog.h>
#include <command.h>
#include <image.h>
#include <asm/byteorder.h>
#include "octeon_boot.h"
#include "octeon_hal.h"
#include "octeon_pci.h"

/* add  flashappimage command to upgrade firmware  */

#include <cfinfo.h>
#include <macinfo.h>
#include <cramfs/cramfs_fs.h>
#include <net.h>
///=====================================================
ulong check_sum(char*, long long);
int atoi_like(char*);
extern ulong writed_num;
extern ulong NetBootFileXferSize;
extern ulong admin;
//=================================================

	
#define IMAGE_CONTENT_MAX	243*CFG_MAX_FLASH_SECT_SIZE // 243 sectors* 128KB = 31104KB
#define	FLASH_END	(FLASHTEST_START-1)
#define SUPER_BLOCK_SIZE 76

int update_md(ManuInfo_t* manuinfo) {

	char cmd[100];
	memset(cmd, 0x0, 100);
	sprintf(cmd, "protect off 0x%x 0x%x", MANUFACTURING_FLASH_START, MANUFACTURING_FLASH_START+CFG_MD_SIZE-1);
	run_command(cmd, 0);

	memset(cmd, 0x0, 100);
	sprintf(cmd, "erase 0x%x 0x%x", MANUFACTURING_FLASH_START, MANUFACTURING_FLASH_START+CFG_MD_SIZE-1);
	run_command(cmd, 0);
		
	memset(cmd, 0x0, 100);
	sprintf(cmd, "cp.b 0x%x 0x%x 0x%x", manuinfo, MANUFACTURING_FLASH_START,CFG_MD_SIZE);
	run_command(cmd, 0);
		
	return 0;
}


int do_flash_upgrade (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    int rcode = 0;
    char cmd[100];
    char *s;
    int fail;
    
    Header* img_info;
    unsigned char* tmp_ptr;
    unsigned int img_chksum;
    
    ManuInfo_t *manuinfo;
    int bootflag;
    
    admin=1;

    ulong tftp_file_size,uboot_flash_size,img_max_size,FLASH_START;
    switch(argc){
    case 0:
    case 1 :
	printf ("Usage:\n%s\n", cmdtp->usage);
	return 1;
    default: 
		if ((s = getenv("serverip")) == NULL) 
			{
			printf("Can't get filesize \n");
		        return 1;
			}
		
		memset(cmd, 0x0, 100);
		sprintf(cmd,"ping %s",s);
		run_command(cmd, 0);

		sprintf (cmd,"tftpboot 0x%x %s",tmp_ram_addr, argv[1]);
		fail=run_command(cmd,0);
		
		if (fail == -1) {
			printf("TFTP failed\n");
			return 1;
		}
		
    if ((s = getenv("filesize")) != NULL) 
        		tftp_file_size = simple_strtoul(s, NULL, 16);
		else
		{
			printf("Can't get filesize \n");
		        return 1;
		}
		if ((s = getenv("uboot_flash_size")) != NULL) 
        		uboot_flash_size = simple_strtoul(s, NULL, 16);
		else
		{
			printf("Can't get uboot_flash_size\n");
		       return 1;  
		}
					 
		
		img_info=(Header*)tmp_ram_addr;
		
		tftp_file_size-=sizeof(Header); // minus header length
		tmp_ptr=(unsigned char*)tmp_ram_addr+sizeof(Header);
		
		printf("Calculating checksum ........\n");
		
		img_chksum=check_sum(tmp_ptr, img_info->size);
		
		//printf("image checksum is %d\n",img_chksum);
		
		if (img_chksum!=img_info->checksum) {
			printf("The checksum of image is incorrect!\n");
		  return 1;			
		} else {
			printf("The checksum of image is correct, writing to flash...........\n");
		}
		
		manuinfo = (ManuInfo_t *)MANUFACTURING_RAM_START;
		memset(manuinfo,0,sizeof(ManuInfo_t));
		memcpy(manuinfo,(uchar *)(MANUFACTURING_FLASH_START), sizeof(ManuInfo_t));
		
		//for dual image
		if ( (manuinfo->currimage)==0x2A) { // image=0 
			bootflag=0;
		FLASH_START=CFG_FLASH_BASE+CFG_FAILSAFE_BOOTLOADER_SIZE+uboot_flash_size;
		} else { //image=1
			bootflag=1;
			FLASH_START = CFG_FLASH_BASE + CFG_FAILSAFE_BOOTLOADER_SIZE + uboot_flash_size + IMAGE_CONTENT_MAX;
		}
			
		manuinfo->image_status[bootflag]=0;
		update_md(manuinfo);
			

		img_max_size = tftp_file_size + (CFG_MAX_FLASH_SECT_SIZE - (tftp_file_size % CFG_MAX_FLASH_SECT_SIZE)); //make it be (times of 128)

			
		if(img_max_size > IMAGE_CONTENT_MAX)
		{
			printf(" *Warning !File size must be smaller than %d byte\n", IMAGE_CONTENT_MAX);
		       return 1;
		}
			
		memset(cmd, 0x0, 100);
		sprintf(cmd, "protect off 0x%x 0x%x", FLASH_START, FLASH_START+img_max_size-1 );
		run_command(cmd, 0);

		memset(cmd, 0x0, 100);
		sprintf(cmd, "erase 0x%x 0x%x", FLASH_START, FLASH_START+img_max_size-1);
		run_command(cmd, 0);
		
		memset(cmd, 0x0, 100);
		sprintf(cmd, "cp.b 0x%x 0x%x 0x%x", tmp_ptr, FLASH_START, img_max_size);
		run_command(cmd, 0);
		
		manuinfo->kernelchecksum[bootflag]=img_chksum;
		manuinfo->kernelsize[bootflag]=tftp_file_size;
		memset(manuinfo->version[bootflag], 0x0, 16);
		memcpy(manuinfo->version[bootflag],img_info->version,16);
		
		//printf("version:%s\n",manuinfo->version[bootflag]);
		
		manuinfo->image_status[bootflag]=1;
		update_md(manuinfo);
		
		return 0;
	}
	return rcode;
}




int do_flashbootloader(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    int rcode = 0;
    char cmd[100];
    char *s;
    int fail;

    ulong tftp_file_size,uboot_flash_size,uboot_start;
    switch(argc){
    case 0:
    case 1 :
	printf ("Usage:\n%s\n", cmdtp->usage);
	return 1;
    default: 
		if ((s = getenv("serverip")) == NULL) 
		{
			printf("Can't get filesize \n");
		        return 1;
		}
	
		memset(cmd, 0x0, 100);
		sprintf(cmd,"ping %s",s);
		run_command(cmd,0);
	  
		sprintf (cmd,"tftpboot 0x%x %s",tmp_ram_addr, argv[1]);
		fail=run_command(cmd,0);
		
		if (fail == -1) {
			printf("TFTP failed\n");
			return 1;
		}
               		          
    		if ((s = getenv("filesize")) != NULL) 
        		tftp_file_size = simple_strtoul(s, NULL, 16);
		else
		{
			printf("Can't get filesize \n");
		        return 1;
		}
		if ((s = getenv("uboot_flash_size")) != NULL) 
        		uboot_flash_size = simple_strtoul(s, NULL, 16);
		else
		{
			printf("Can't get uboot_flash_size\n");
		        return 1;
		}
			
		if (tftp_file_size > uboot_flash_size ) {
			printf("Error bootloader file! \n");
		       return 1;
    		}
			
    uboot_start = CFG_FLASH_BASE+CFG_FAILSAFE_BOOTLOADER_SIZE;
			
		memset(cmd, 0x0, 100);
		sprintf(cmd, "protect off 0x%x 0x%x", uboot_start, uboot_start+uboot_flash_size-1 );
		run_command(cmd, 0);


		memset(cmd, 0x0, 100);
		sprintf(cmd, "erase 0x%x 0x%x", uboot_start, uboot_start+uboot_flash_size-1);
		run_command(cmd, 0);
		
		memset(cmd, 0x0, 100);
		sprintf(cmd, "cp.b 0x%x 0x%x 0x%x", tmp_ram_addr, uboot_start, tftp_file_size);
		run_command(cmd, 0);
		return 0;
	}
	return rcode;
}



ulong  check_sum(char* image_addr,long long length)
{
	ulong i=0,checksum = 0;	
	unsigned long mask;
	mask = 0x000000ff;
	while (i < length){
			checksum+= ((*image_addr) & mask);		// only get last one byte
			image_addr++;
			i++;
		}
	return checksum;	
}

int do_set_bootflag (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
       int rcode=0;
	int  image_number=0;
	char	cmd[100];
	BYTE currimage;
	ManuInfo_t *manuinfo;
	ulong pre_value;
	switch(argc){
    	case 0:
	case 1:
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
    	default :
		image_number= atoi_like(argv[1]);
		if ((image_number<0)||(image_number>1)) {
			printf("Boot flag must be 0 or 1\n");
			return 1;	}

		manuinfo = (ManuInfo_t *)MANUFACTURING_RAM_START;
		memset(manuinfo,0,sizeof(ManuInfo_t));
		memcpy(manuinfo,(uchar *)(MANUFACTURING_FLASH_START), sizeof(ManuInfo_t));

		if (image_number==0)
			currimage=0x2A;  // image 0
		else 
			currimage=0x2B;  // image 1

		manuinfo->currimage=currimage;

	       if (admin==1)
			pre_value=1;
		else
			pre_value=0;
		admin=1;
		memset(cmd, 0x0, 100);
		sprintf(cmd, "protect off 0x%x 0x%x", MANUFACTURING_FLASH_START, MANUFACTURING_FLASH_START+CFG_MD_SIZE-1);
		run_command(cmd, 0);

		
		memset(cmd, 0x0, 100);
		sprintf(cmd, "erase 0x%x 0x%x", MANUFACTURING_FLASH_START, MANUFACTURING_FLASH_START+CFG_MD_SIZE-1);
		run_command(cmd, 0);
		
		memset(cmd, 0x0, 100);
		sprintf(cmd, "cp.b 0x%x 0x%x 0x%x", manuinfo, MANUFACTURING_FLASH_START,CFG_MD_SIZE);
		run_command(cmd, 0);

   		 memset(cmd,0x0,100);
		if (image_number==0)
		{
			printf("Active image is image %d\n",0);
			//sprintf(cmd,"bootoctlinux 0x%x root=/dev/cfa5 rw",tmp_ram_addr);
			//setenv("bootcmd",cmd);
		}
	       else
	       	{
			printf("Active image is image %d\n",1);
			//sprintf(cmd,"bootoctlinux 0x%x root=/dev/cfa7 rw",tmp_ram_addr);
			//setenv("bootcmd",cmd);
		}
		if(pre_value==0)
			admin=0;
		return 0;
	}
	return rcode;
}

int do_dualimageboot(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{

    int rcode = 0;
    int bootflag;
    int tries=0;
    char cmd1[200];
    char *s,*s1,*s2,*s3;
    char *s4;
    ManuInfo_t *manuinfo;
    ulong uboot_flash_size,FLASH_START;
    ulong cal_chksum;

    ulong kernel_gz_padded_size;
    struct cramfs_super* super;

	admin=1;
		//read md
		manuinfo = (ManuInfo_t *)MANUFACTURING_RAM_START;
		memset(manuinfo,0,sizeof(ManuInfo_t));
		memcpy(manuinfo,(uchar *)(MANUFACTURING_FLASH_START), sizeof(ManuInfo_t));

		if ((s3 = getenv("uboot_flash_size")) != NULL) 
        		uboot_flash_size = simple_strtoul(s3, NULL, 16);
		else
		{
			printf("Can't get uboot_flash_size\n");
		        return 1;
		}

retry:
		
		if ( (manuinfo->currimage)==0x2A) { // image=0 
			bootflag=0;
		FLASH_START=CFG_FLASH_BASE+CFG_FAILSAFE_BOOTLOADER_SIZE+uboot_flash_size;
		} else { //image=1
			bootflag=1;
			FLASH_START = CFG_FLASH_BASE + CFG_FAILSAFE_BOOTLOADER_SIZE + uboot_flash_size + IMAGE_CONTENT_MAX;
		}
		
		if (manuinfo->image_status[bootflag]!= 1){
			if (tries==0) {
				printf("status of image %d error!! switch to another image... \n", bootflag);
				
				if (manuinfo->currimage == 0x2A ) manuinfo->currimage = 0x2B;
				else 	manuinfo->currimage = 0x2A;
				
				update_md(manuinfo);
				tries++;
				goto retry;
			} else {			
				printf("Two images are dead!\n");
				return 1;
			}
		}
		
		cal_chksum=check_sum(FLASH_START, manuinfo->kernelsize[bootflag]);

		if (cal_chksum != manuinfo->kernelchecksum[bootflag]) {
			printf("Checksum error!!\n");
			return 1;
		}
		
		super=(struct cramfs_super*) FLASH_START;
		
		kernel_gz_padded_size=(super->root).offset*4 -SUPER_BLOCK_SIZE; // actual offset of cramfs should multiply 4

		printf("Uncompressing kernel ... \n");

		memset(cmd1, 0x0, 100);
		sprintf(cmd1, "cp.b 0x%x 0x%x 0x%x", FLASH_START+SUPER_BLOCK_SIZE, decompress_addr1, kernel_gz_padded_size);
		run_command(cmd1, 0);
		
		memset(cmd1,0x0,100);
    sprintf(cmd1, "gunzip %x %x %qx %x", decompress_addr1, kernel_gz_padded_size, tmp_ram_addr, IMAGE_CONTENT_MAX); //decompress_len+1
		run_command(cmd1, 0);	
				

		memset(cmd1, 0x0, 200);
		sprintf(cmd1,"bootoctlinux 0x%x  console=ttyS0,115200",tmp_ram_addr);
		run_command(cmd1, 0);
		
	return rcode;
}


ulong do_show_bootflag (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	ManuInfo_t *manuinfo;
		
	manuinfo=MANUFACTURING_FLASH_START;
	
	if ( (manuinfo->currimage)==0x2A) 
	       printf("Active image is image 0\n");
       else 
       	        printf("Active image is image 1\n");
	return 0;
}

int atoi_like(char *str)
{

        int  i=0,len=0,num=0;
       
        len=strlen(str);

        for ( i=0; i < len; i++ )
        {
                if(str[i]!='-')
                {
                  num*=10;
                  num+=str[i]-'0';
                }
                if((str[i]=='-')&&(i!=0))
                  break;
        }
	 if (str[0]=='-')
         return (num*-1);
        else
         return num;
}

int input_passwd()
{
	printf("Permission denied.\n");
	return 1;
}




U_BOOT_CMD(
	setbootflag,  2,  1,  do_set_bootflag,
	"setbootflag <image number | 0 or 1>\n",
	"-set boot flag to indicate the preferred image\n"
	 " \n"
        "\t setbootflag <image number | 0 or 1>\n"
);	
U_BOOT_CMD(
	showbootflag, 1,  1,  do_show_bootflag,
	"show boot flag to indicate the preferred image\n",
	" -show boot flag to indicate the preferred image\n"
	"\n"	
);



U_BOOT_CMD(
	flashappimage,  2 , 2,  do_flash_upgrade,
	"flashappimage -Using this command to upgrade firmware \n",
	" \n"
	"flashappimage <image file name>\n"
);

U_BOOT_CMD(
	flashbootloader,  2 , 2,  do_flashbootloader,
	"flashbootloader -Using this command to upgrade bootloader\n",
	" \n"
	"flashbootloader <bootloader file name>\n"
);

U_BOOT_CMD(
	dualimageboot,      1,      0,      do_dualimageboot,
	"dualimageboot - Boot from a linux ELF image with preferred image\n",
	NULL
);
