/***********************license start************************************
 * Copyright (c) 2005-2007  Cavium Networks (support@cavium.com). 
 * All rights reserved.
 * 
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 * 
 *     * Neither the name of Cavium Networks nor the names of
 *       its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written
 *       permission.  
 * 
 * This Software, including technical data, may be subject to U.S.  export 
 * control laws, including the U.S.  Export Administration Act and its 
 * associated regulations, and may be subject to export or import regulations 
 * in other countries.  You warrant that You will comply strictly in all 
 * respects with all such regulations and acknowledge that you have the 
 * responsibility to obtain licenses to export, re-export or import the 
 * Software.  
 * 
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS" 
 * AND WITH ALL FAULTS AND CAVIUM NETWORKS MAKES NO PROMISES, REPRESENTATIONS 
 * OR WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH 
 * RESPECT TO THE SOFTWARE, INCLUDING ITS CONDITION, ITS CONFORMITY TO ANY 
 * REPRESENTATION OR DESCRIPTION, OR THE EXISTENCE OF ANY LATENT OR PATENT 
 * DEFECTS, AND CAVIUM SPECIFICALLY DISCLAIMS ALL IMPLIED (IF ANY) WARRANTIES 
 * OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR 
 * PURPOSE, LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET 
 * POSSESSION OR CORRESPONDENCE TO DESCRIPTION.  THE ENTIRE RISK ARISING OUT 
 * OF USE OR PERFORMANCE OF THE SOFTWARE LIES WITH YOU.
 * 
 ***********************license end**************************************/

/**
 *
 * $Id: octeon_eth.c,v 1.2.8.2 2009/08/24 02:50:01 david_yu Exp $
 * 
 */
 
 
#include <common.h>
#include <command.h>
#include <exports.h>
#include <linux/ctype.h>
#include <lib_octeon.h>
#include <octeon_eeprom_types.h>
#include <net.h>
#include <miiphy.h>
#include "octeon_boot.h"
#include "octeon_eeprom_types.h"
#include "lib_octeon_shared.h"
#include "cvmx-config.h"
#include "cvmx-wqe.h"
#include "cvmx-pow.h"
#include "cvmx-pko.h"
#include "cvmx-ipd.h"
#include "cvmx-pip.h"
#include "cvmx-spi.h"
#include "cvmx-mdio.h"
#include "cvmx-helper.h"
#include "cvmx-mgmt-port.h"
#include "cvmx-helper-util.h"
#include "cvmx-bootmem.h"

#define DEBUG 1
#undef  DEBUG

#ifdef DEBUG
#define dprintf printf
#else
#define dprintf(...)
#endif

/* Global flag indicating common hw has been set up */
int octeon_global_hw_inited = 0;

/* pointer to device to use for loopback.  Only needed for IPD reset workaround */
struct eth_device *loopback_dev;

int octeon_eth_send(struct eth_device *dev, volatile void *packet, int length);
int octeon_ebt3000_rgmii_init(struct eth_device *dev, bd_t * bis);
void octeon_eth_rgmii_enable(struct eth_device *dev);


/************************************************************************/
/* Ethernet device private data structure for octeon ethernet */
typedef struct
{
    uint32_t port;
    uint32_t interface;
    uint32_t queue;
    uint32_t initted_flag;
    uint32_t link_speed:2;   /* current link status, use to reconfigure on status changes */
    uint32_t link_duplex:1;
    uint32_t link_status:1;
    uint32_t loopback:1;
    uint32_t enabled:1;
} octeon_eth_info_t;

#if  defined(OCTEON_INTERNAL_ENET) || defined(OCTEON_MGMT_ENET)

/* Variable shared between ethernet routines. */
static int card_number;
static int available_mac_count = -1;
static uint8_t mac_addr[6];
static uint64_t port_state[50];  /* Save previous link state so we only configure/print on transitions */


#define USE_HW_TCPUDP_CHECKSUM  0

/* Make sure that we have enough buffers to keep prefetching blocks happy.
** Absolute minimum is probably about 200. */
#define NUM_PACKET_BUFFERS  1000

#define PKO_SHUTDOWN_TIMEOUT_VAL     100
/****************************************************************/


static void cvm_oct_fill_hw_memory(uint64_t pool, uint64_t size, uint64_t elements)
{

    int64_t memory;
    static int alloc_count = 0;
    char tmp_name[64];

    dprintf("cvm_oct_fill_hw_memory: pool: 0x%qx, size: 0xx%qx, count: 0x%qx\n", pool, size, elements);
    sprintf(tmp_name, "%s_fpa_alloc_%d", OCTEON_BOOTLOADER_NAMED_BLOCK_TMP_PREFIX, alloc_count++);
    memory = cvmx_bootmem_phy_named_block_alloc(size * elements, 0, 0x40000000, 128, tmp_name, CVMX_BOOTMEM_FLAG_END_ALLOC);
    if (memory < 0)
    {
        printf("Unable to allocate %lu bytes for FPA pool %ld\n", elements*size, pool);
        return;
    }
    while (elements--)
    {
        cvmx_fpa_free((void *)(uint32_t)(memory + elements*size), pool, 0);
    }
}


/**
 * Set the hardware MAC address for a device
 *
 * @param interface    interface of port to set
 * @param index    index of port to set MAC address for
 * @param addr   Address structure to change it too. 
 * @return Zero on success
 */
static int cvm_oct_set_mac_address(int interface, int index, void *addr)
{
    cvmx_gmxx_prtx_cfg_t gmx_cfg;


    int i;
    uint8_t *ptr = addr;
    uint64_t mac = 0;
    for (i=0; i<6; i++)
        mac = (mac<<8) | (uint64_t)(ptr[i]);

    /* Disable interface */
    gmx_cfg.u64 = cvmx_read_csr(CVMX_GMXX_PRTX_CFG(index, interface));
    cvmx_write_csr(CVMX_GMXX_PRTX_CFG(index, interface), gmx_cfg.u64 & ~1ull);

    cvmx_write_csr(CVMX_GMXX_SMACX(index, interface), mac);
    cvmx_write_csr(CVMX_GMXX_RXX_ADR_CAM0(index, interface), ptr[0]);
    cvmx_write_csr(CVMX_GMXX_RXX_ADR_CAM1(index, interface), ptr[1]);
    cvmx_write_csr(CVMX_GMXX_RXX_ADR_CAM2(index, interface), ptr[2]);
    cvmx_write_csr(CVMX_GMXX_RXX_ADR_CAM3(index, interface), ptr[3]);
    cvmx_write_csr(CVMX_GMXX_RXX_ADR_CAM4(index, interface), ptr[4]);
    cvmx_write_csr(CVMX_GMXX_RXX_ADR_CAM5(index, interface), ptr[5]);


    cvmx_gmxx_rxx_adr_ctl_t control;
    control.u64 = 0;
    control.s.bcst = 1;     /* Allow broadcast MAC addresses */
    control.s.mcst = 1; /* Force reject multicat packets */
    control.s.cam_mode = 1; /* Filter packets based on the CAM */
    cvmx_write_csr(CVMX_GMXX_RXX_ADR_CTL(index, interface), control.u64);

    cvmx_write_csr(CVMX_GMXX_RXX_ADR_CAM_EN(index, interface), 1);


    /* Return interface to previous enable state */
    cvmx_write_csr(CVMX_GMXX_PRTX_CFG(index, interface), gmx_cfg.u64);

    return 0;
}

/**
 * Configure common hardware for all interfaces
 */
static void cvm_oct_configure_common_hw(void)
{

    if (getenv("disable_spi"))
    {

        /* Do this here so that we can disabled this after boot but before
        ** we use networking.
        */
        cvmx_spi_callbacks_t spi_callbacks_struct;
        cvmx_spi_get_callbacks(&spi_callbacks_struct);
        spi_callbacks_struct.reset_cb            = 0;
        spi_callbacks_struct.calendar_setup_cb   = 0;
        spi_callbacks_struct.clock_detect_cb     = 0;
        spi_callbacks_struct.training_cb         = 0;
        spi_callbacks_struct.calendar_sync_cb    = 0;
        spi_callbacks_struct.interface_up_cb     = 0;
        printf("SPI interface disabled with 'disable_spi' environment variable\n");
        cvmx_spi_set_callbacks(&spi_callbacks_struct);
    }
    /* Setup the FPA */
    cvmx_fpa_enable();

    cvm_oct_fill_hw_memory(CVMX_FPA_WQE_POOL, CVMX_FPA_WQE_POOL_SIZE, NUM_PACKET_BUFFERS);
#if CVMX_FPA_OUTPUT_BUFFER_POOL != CVMX_FPA_PACKET_POOL
    cvm_oct_fill_hw_memory(CVMX_FPA_OUTPUT_BUFFER_POOL, CVMX_FPA_OUTPUT_BUFFER_POOL_SIZE, 128);
#endif
    cvm_oct_fill_hw_memory(CVMX_FPA_PACKET_POOL, CVMX_FPA_PACKET_POOL_SIZE, NUM_PACKET_BUFFERS);

    cvmx_helper_initialize_packet_io_global();
    cvmx_helper_initialize_packet_io_local();

    /* Set POW get work timeout to maximum value */
    cvmx_write_csr(CVMX_POW_NW_TIM, 0x3ff);
}

/**
 * Packet transmit
 *
 * @param skb    Packet to send
 * @param dev    Device info structure
 * @return Always returns zero
 */
static int cvm_oct_xmit(struct eth_device *dev, void *packet, int len)
{
    octeon_eth_info_t* priv = (octeon_eth_info_t*)dev->priv;
    cvmx_pko_command_word0_t    pko_command;
    cvmx_buf_ptr_t              hw_buffer;


    dprintf("cvm_oct_xmit addr: %p, len: %d\n", packet, len);


    /* Build the PKO buffer pointer */
    hw_buffer.u64 = 0;
    hw_buffer.s.addr = cvmx_ptr_to_phys(packet);
    hw_buffer.s.pool = CVMX_FPA_PACKET_POOL;
    hw_buffer.s.size = CVMX_FPA_PACKET_POOL_SIZE;

    /* Build the PKO command */
    pko_command.u64 = 0;
    pko_command.s.subone0 = 1;
    pko_command.s.dontfree = 0;
    pko_command.s.segs = 1;
    pko_command.s.total_bytes = len;
    /* Send the packet to the output queue */

    cvmx_pko_send_packet_prepare(priv->port, priv->queue, 0);
    if (cvmx_pko_send_packet_finish(priv->port, priv->queue, pko_command, hw_buffer, 0))
    {
        printf("Failed to send the packet\n");
    }
	return 0;
}

/**
 * Configure the RGMII port for the negotiated speed
 *
 * @param dev    Linux device for the RGMII port
 */
static void cvm_oct_configure_rgmii_speed(struct eth_device *dev)
{
    octeon_eth_info_t* priv = (octeon_eth_info_t*)dev->priv;

    cvmx_helper_link_info_t link_state = cvmx_helper_link_get(priv->port);

    if (link_state.u64 != port_state[priv->port])
    {

        printf("%s: ", dev->name);
        if (!link_state.s.link_up)
            printf("Down ");
        else
        {
            printf("Up ");
            printf("%d Mbps ", link_state.s.speed);
            if (link_state.s.full_duplex)
                printf("Full duplex ");
            else
                printf("Half duplex ");
        }
        printf("(port %2d)\n", priv->port);
        cvmx_helper_link_set(priv->port, link_state);
        port_state[priv->port] = link_state.u64;
    }
}
int octeon_network_hw_shutdown_internal(void)
{
    int port, queue;
    int retval = 0;
    int i;
    int num_interfaces;

    num_interfaces = cvmx_helper_get_number_of_interfaces();
    if (num_interfaces > 2)
        num_interfaces = 2;

    octeon_eth_info_t *oct_eth_info = NULL;

    /* Don't need to do anything if networking not used */
    if (!octeon_global_hw_inited)
        return(retval);

    if (octeon_is_model(OCTEON_CN38XX_PASS1))
    {
        printf("\n");
        printf("\n");
        printf("ERROR: TFTP network reset not supported on PASS 1\n");
        printf("\n");
        return -1;
    }

    /* For pass 2 we need to make sure the device that we selected to loop back packets
    ** on has been initialized */
    if (octeon_is_model(OCTEON_CN38XX_PASS2))
    {
        if (!loopback_dev)
        {
            printf("Neither interface is RGMII, unable to reset network HW\n");
            return -1;
        }
        oct_eth_info = (octeon_eth_info_t *)loopback_dev->priv;
        dprintf("using port %d to loopback packets for ipd workaround\n", oct_eth_info->port);
    }


    /* Disable any further packet reception */
    for (i = 0; i < num_interfaces; i++)
    {
        if (CVMX_HELPER_INTERFACE_MODE_RGMII == cvmx_helper_interface_get_mode(i)
            || CVMX_HELPER_INTERFACE_MODE_GMII == cvmx_helper_interface_get_mode(i)
            || CVMX_HELPER_INTERFACE_MODE_SGMII == cvmx_helper_interface_get_mode(i)
            || CVMX_HELPER_INTERFACE_MODE_XAUI == cvmx_helper_interface_get_mode(i))
        {
            cvmx_write_csr(CVMX_ASXX_RX_PRT_EN(i), 0x0);   /* Disable the RGMII RX ports */
            cvmx_write_csr(CVMX_ASXX_TX_PRT_EN(i), 0x0);   /* Disable the RGMII TX ports */
            int num_ports = cvmx_helper_ports_on_interface(i);
            int j;
            for (j = 0; j < num_ports;j++)
            {
                /* Disable MAC filtering */
                cvmx_gmxx_prtx_cfg_t gmx_cfg;
                gmx_cfg.u64 = cvmx_read_csr(CVMX_GMXX_PRTX_CFG(j, i));
                cvmx_write_csr(CVMX_GMXX_PRTX_CFG(j, i), gmx_cfg.u64 & ~1ull);
                cvmx_write_csr(CVMX_GMXX_RXX_ADR_CTL(j, i), 1);
                cvmx_write_csr(CVMX_GMXX_RXX_ADR_CAM_EN(j, i), 0);
                cvmx_write_csr(CVMX_GMXX_PRTX_CFG(j, i), gmx_cfg.u64);
            }
        }
    }
    /* TODO: If SPI interface is used, the SPI device should be configured to stop receiving packets*/


    /* Make sure PKO is empty.  Read doorbell count
    ** for configured output queues.  We at most have configured
    ** ports 0-9, 16-25 */
    int timeout;
    uint32_t count;
    for (i = 0; i < num_interfaces; i++)
    {
        int ports_per_interface = cvmx_helper_ports_on_interface(i);
        for (port = i*16; port < i*16 + ports_per_interface; port++)
        {
            queue = cvmx_pko_get_base_queue(port);
            timeout = PKO_SHUTDOWN_TIMEOUT_VAL;
            /* Check doorbell count */
            while (0 != cvmx_cmd_queue_length(CVMX_CMD_QUEUE_PKO(queue)) && timeout-- > 0)
                udelay(10000);
            if ((count = cvmx_cmd_queue_length(CVMX_CMD_QUEUE_PKO(queue))))
            {
                printf("Error draining PKO: queue %d doorbell count: %d is not zero\n", queue, count);
                retval = -1;
            }

        }
    }

    /* Drain POW, free buffers to FPA in case they are needed for loopback workaround. */
    cvmx_wqe_t *work;
    while ((work = cvmx_pow_work_request_sync(1)))
    {
        cvmx_helper_free_packet_data(work);
        cvmx_fpa_free(work, CVMX_FPA_WQE_POOL, 0);
        dprintf("Freeing packet in network shutdown!\n");
    }

    /* In Octeon CN38XX, the IPD reset did not reset a fifo ptr,
    ** so this is set to zero by using an RGMII loopback.  If neither interface
    ** is RGMII, then this workaround won't work.
    ** All other chips don't need this step - the reset is sufficient.
    */
    if (octeon_is_model(OCTEON_CN38XX_PASS2))
    {
        oct_eth_info = (octeon_eth_info_t *)loopback_dev->priv;

        /* Set loopback_dev port into loopback mode*/
        cvmx_helper_rgmii_internal_loopback(oct_eth_info->port);

        /* Now read IPD_PTR_COUNT register */

        cvmx_ipd_ptr_count_t ipd_cnt;
        ipd_cnt.u64 = cvmx_read_csr(CVMX_IPD_PTR_COUNT);
        int to_add = (ipd_cnt.s.wqev_cnt + ipd_cnt.s.wqe_pcnt) & 0x7;
        int ptr = (-to_add) & 0x7;

        dprintf("Ptr is %d, to add count is: %d\n", ptr, to_add);

        /* Send enough packets to wrap internal ipd pointer to 0 */
        while (to_add--)
        {
            char buffer[100];
            /* Send a packet through loopback to be processed by IPD */
            octeon_eth_send(loopback_dev, buffer, 100);
            udelay(50000);
        }

        udelay(200000);

        /* Check pointer to make sure that our loopback packets moved it to 0 */
        ipd_cnt.u64 = cvmx_read_csr(CVMX_IPD_PTR_COUNT);
        to_add = (ipd_cnt.s.wqev_cnt + ipd_cnt.s.wqe_pcnt) & 0x7;
        ptr = (-to_add) & 0x7;
        dprintf("(After) Ptr is %d, to add count is: %d\n", ptr, to_add);
        if (to_add)
        {
            printf("ERROR: unable to reset packet input!\n");
            return -1;
        }

        /* Disable loopback and leave port disabled. */
        cvmx_write_csr(CVMX_ASXX_RX_PRT_EN(oct_eth_info->interface), 0);
        cvmx_write_csr(CVMX_ASXX_TX_PRT_EN(oct_eth_info->interface), 0);
        cvmx_write_csr(CVMX_ASXX_PRT_LOOP(oct_eth_info->interface), 0);
        oct_eth_info->loopback = 0;
        cvmx_read_csr(CVMX_MIO_BOOT_BIST_STAT);  /* Make sure writes are complete */
        /* End workaround */
    }



    /* Drain POW  */
    while ((work = cvmx_pow_work_request_sync(1)))
    {
        cvmx_helper_free_packet_data(work);
        cvmx_fpa_free(work, CVMX_FPA_WQE_POOL, 0);
        dprintf("Freeing packet in network shutdown!\n");
    }



    /* Drain FPA - this may not be necessary */
    for (i = 0; i < 8; i++)
    {
        count = 0;
        while (cvmx_fpa_alloc(i))
            count++;
        dprintf("Freed %d buffers from pool %d\n", count, i);
    }


    /* Since we are ingoring flow control, we assume that if PKO is empty
    ** the RGMII/SPI buffers are empty by now.  Since all we did was DHCP/TFTP,
    ** we should not have any outstanding packets at this point.
    ** In other situations, the GMX/SPX blocks would need to be checked to make sure they are
    ** empty before resetting.
    */
    /* Delay to let any data in GMX/SPX drain */
    udelay(50000);

    /* Reset the FPA, IPD, PKO.  These bits are new
    ** in pass 2 silicon. */
    cvmx_ipd_ctl_status_t ipd_ctl_status;
    ipd_ctl_status.u64 = cvmx_read_csr(CVMX_IPD_CTL_STATUS);
    ipd_ctl_status.s.reset = 1;
    cvmx_write_csr(CVMX_IPD_CTL_STATUS, ipd_ctl_status.u64);

    if (!octeon_is_model(OCTEON_CN38XX_PASS2))
    {
        cvmx_pip_sft_rst_t pip_reset;
        pip_reset.u64 = cvmx_read_csr(CVMX_PIP_SFT_RST);
        pip_reset.s.rst = 1;
        cvmx_write_csr(CVMX_PIP_SFT_RST, pip_reset.u64);
    }

    cvmx_pko_reg_flags_t pko_reg_flags;
    pko_reg_flags.u64 = cvmx_read_csr(CVMX_PKO_REG_FLAGS);
    pko_reg_flags.s.reset = 1;
    cvmx_write_csr(CVMX_PKO_REG_FLAGS, pko_reg_flags.u64);

    cvmx_fpa_ctl_status_t fpa_ctl_status;
    fpa_ctl_status.u64 = cvmx_read_csr(CVMX_FPA_CTL_STATUS);
    fpa_ctl_status.s.reset = 1;
    cvmx_write_csr(CVMX_FPA_CTL_STATUS, fpa_ctl_status.u64);

    return(retval);
}

int octeon_network_hw_shutdown(void)
{
    int retval = octeon_network_hw_shutdown_internal();

    /* Free command queue named block, as we have reset the hardware */
    cvmx_bootmem_phy_named_block_free("cvmx_cmd_queues", 0);
    if (retval < 0)
    {
        /* Make this a fatal error since the error message is easily missed and ignoring
        ** it can lead to very strange networking behavior in the application.
        */
        printf("FATAL ERROR: Network shutdown failed.  Please reset the board.\n");
        while (1)
            ;
    }
    return(retval);
}

/*******************  Octeon networking functions ********************/
int octeon_ebt3000_rgmii_init(struct eth_device *dev, bd_t * bis)         /* Initialize the device	*/
{
    octeon_eth_info_t* priv = (octeon_eth_info_t*)dev->priv;
    int interface = priv->interface;
    dprintf("octeon_ebt3000_rgmii_init(), dev_ptr: %p, port: %d, queue: %d\n", dev, priv->port, priv->queue);

    if (priv->initted_flag)
        return 1;

    if (!octeon_global_hw_inited)
    {
        cvm_oct_configure_common_hw();
    }

    /* Ignore backpressure on RGMII ports */
    cvmx_write_csr(CVMX_GMXX_TX_OVR_BP(interface), 0xf << 8 | 0xf);

    cvm_oct_set_mac_address(interface, interface & 0x3, dev->enetaddr);

    if (!octeon_global_hw_inited)
    {
        cvmx_helper_ipd_and_packet_input_enable();
        octeon_global_hw_inited = 1;
    }
    priv->initted_flag = 1;

    if (simple_strtoul (getenv("rxclk0"), NULL, 16) == 0x0)
        cvmx_write_csr(CVMX_ASXX_RX_CLK_SETX(0, interface), 0x0);
    else {
        cvmx_write_csr(CVMX_ASXX_RX_CLK_SETX(0, interface), simple_strtoul (getenv("rxclk0"), NULL, 16));
        dprintf("CVMX_ASXX_RX_CLK_SETX(0) = 0x%x\n", simple_strtoul (getenv("rxclk0"), NULL, 16));
    }

    if (simple_strtoul (getenv("rxclk1"), NULL, 16) == 0x0)
        cvmx_write_csr(CVMX_ASXX_RX_CLK_SETX(1, interface), 0x18);
    else {
        cvmx_write_csr(CVMX_ASXX_RX_CLK_SETX(1, interface), simple_strtoul (getenv("rxclk1"), NULL, 16));
        dprintf("CVMX_ASXX_RX_CLK_SETX(0) = 0x%x\n", simple_strtoul (getenv("rxclk1"), NULL, 16));
    }
    
    if (simple_strtoul (getenv("txclk0"), NULL, 16) == 0x0)
        cvmx_write_csr(CVMX_ASXX_TX_CLK_SETX(0, interface), 0x19);
    else {
        cvmx_write_csr(CVMX_ASXX_TX_CLK_SETX(0, interface), simple_strtoul (getenv("txclk0"), NULL, 16));
        dprintf("CVMX_ASXX_TX_CLK_SETX = 0x%x\n", simple_strtoul (getenv("txclk0"), NULL, 16));
    }

    if (simple_strtoul (getenv("txclk1"), NULL, 16) == 0x0)
        cvmx_write_csr(CVMX_ASXX_TX_CLK_SETX(1, interface), 0x18);
    else {
        cvmx_write_csr(CVMX_ASXX_TX_CLK_SETX(1, interface), simple_strtoul (getenv("txclk1"), NULL, 16));
        dprintf("CVMX_ASXX_TX_CLK_SETX = 0x%x\n", simple_strtoul (getenv("txclk1"), NULL, 16));
    }
	
    return(1);
}




#ifdef OCTEON_SPI4000_ENET
int octeon_spi4000_init(struct eth_device *dev, bd_t * bis)         /* Initialize the device	*/
{
    octeon_eth_info_t* priv = (octeon_eth_info_t*)dev->priv;
    static char spi4000_inited[2] = {0};
    dprintf("octeon_spi4000_init(), dev_ptr: %p, port: %d, queue: %d\n", dev, priv->port, priv->queue);


    if (priv->initted_flag)
        return 1;

    if (!octeon_global_hw_inited)
    {
        cvm_oct_configure_common_hw();
    }

    if (!spi4000_inited[priv->interface])
    {
        spi4000_inited[priv->interface] = 1;
        if (__cvmx_helper_spi_enable(priv->interface) < 0)
        {
            printf("ERROR initializing spi4000 on Octeon Interface %d\n", priv->interface);
            return 0;
        }
    }

    if (!octeon_global_hw_inited)
    {
        cvmx_helper_ipd_and_packet_input_enable();
        octeon_global_hw_inited = 1;
    }
    priv->initted_flag = 1;
    return(1);
}
#endif




void octeon_eth_rgmii_enable(struct eth_device *dev)
{
    octeon_eth_info_t *oct_eth_info;
    oct_eth_info = (octeon_eth_info_t *)dev->priv;

    if (CVMX_HELPER_INTERFACE_MODE_RGMII == cvmx_helper_interface_get_mode(oct_eth_info->interface)
        || CVMX_HELPER_INTERFACE_MODE_GMII == cvmx_helper_interface_get_mode(oct_eth_info->interface))
    {
        uint64_t tmp;
        tmp = cvmx_read_csr(CVMX_ASXX_RX_PRT_EN(oct_eth_info->interface));
        tmp |= (1ull << (oct_eth_info->port & 0x3));
        cvmx_write_csr(CVMX_ASXX_RX_PRT_EN(oct_eth_info->interface), tmp);
        tmp = cvmx_read_csr(CVMX_ASXX_TX_PRT_EN(oct_eth_info->interface));
        tmp |= (1ull << (oct_eth_info->port & 0x3));
        cvmx_write_csr(CVMX_ASXX_TX_PRT_EN(oct_eth_info->interface), tmp);
    }


}




/* Keep track of packets sent on each interface so we can
** autonegotiate before we send the first one */
int packets_sent;  
int octeon_eth_send(struct eth_device *dev, volatile void *packet, int length)	   /* Send a packet	*/
{
    dprintf("ethernet TX! ptr: %p, len: %d\n", packet, length);
    /* We need to copy this to a FPA buffer, then give that to TX */

    /* This is needed since we disable rx/tx in halt */
    octeon_eth_rgmii_enable(dev);
    if (packets_sent++ == 0)
        cvm_oct_configure_rgmii_speed(dev);

    void *fpa_buf = cvmx_fpa_alloc(CVMX_FPA_PACKET_POOL);
    if (!fpa_buf)
    {
        printf("ERROR allocating buffer for packet!\n");
    }
    memcpy(fpa_buf, (void *)packet, length);
    cvm_oct_xmit(dev, fpa_buf, length);
    return(0);
}

int octeon_eth_recv(struct eth_device *dev)         /* Check for received packets	*/
{
    cvmx_wqe_t *work = cvmx_pow_work_request_sync(1);
    if (!work)
    {
        /* Poll for link status change, only if work times out.
        ** On some interfaces this is really slow.
        ** getwork timeout is set to maximum. */
        cvm_oct_configure_rgmii_speed(dev);
        return(0);
    }
    if (work->word2.s.rcv_error)
    {
        /* Work has error, so drop */
        printf("Error packet received, dropping\n");
        cvmx_helper_free_packet_data(work);
        cvmx_fpa_free(work, CVMX_FPA_WQE_POOL, 0);
        return(0);
    }


    void *packet_data = cvmx_phys_to_ptr(work->packet_ptr.u64 & 0xffffffffffull);
    int length = work->len;

    dprintf("############# got work: %p, len: %d, packet_ptr: %p\n",
            work, length, packet_data);
#if defined(DEBUG) && 0
    /* Dump out packet contents */
    {
	int i, j;
	unsigned char *up = packet_data;

	for (i = 0; (i + 16) < length; i += 16)
	{
	    printf("%04x ", i);
	    for (j = 0; j < 16; ++j)
	    {
		printf("%02x ", up[i+j]);
	    }
	    printf("    ");
	    for (j = 0; j < 16; ++j)
	    {
		printf("%c", ((up[i+j] >= ' ') && (up[i+j] <= '~')) ? up[i+j] : '.');
	    }
	    printf("\n");
	}
	printf("%04x ", i);
	for (j = 0; i+j < length; ++j)
	{
	    printf("%02x ", up[i+j]);
	}
	for (; j < 16; ++j)
	{
	    printf("   ");
	}
	printf("    ");
	for (j = 0; i+j < length; ++j)
	{
	    printf("%c", ((up[i+j] >= ' ') && (up[i+j] <= '~')) ? up[i+j] : '.');
	}
	printf("\n");
    }
#endif

    NetReceive (packet_data, length);
    cvmx_helper_free_packet_data(work);
    cvmx_fpa_free(work, CVMX_FPA_WQE_POOL, 0);
    /* Free WQE and packet data */
    return(length);
}
void octeon_eth_halt(struct eth_device *dev)			/* stop SCC			*/
{
    /* Disable reception on this port at the GMX block */
    octeon_eth_info_t *oct_eth_info;

    oct_eth_info = (octeon_eth_info_t *)dev->priv;

    if (CVMX_HELPER_INTERFACE_MODE_RGMII == cvmx_helper_interface_get_mode(oct_eth_info->interface)
        || CVMX_HELPER_INTERFACE_MODE_GMII == cvmx_helper_interface_get_mode(oct_eth_info->interface))
    {
        int index = oct_eth_info->interface & 0x3;
        uint64_t tmp;
        cvmx_gmxx_prtx_cfg_t gmx_cfg;
        tmp = cvmx_read_csr(CVMX_ASXX_RX_PRT_EN(oct_eth_info->interface));
        tmp &= ~(1ull << (oct_eth_info->port & 0x3));
        cvmx_write_csr(CVMX_ASXX_RX_PRT_EN(oct_eth_info->interface), tmp);   /* Disable the RGMII RX ports */
        tmp = cvmx_read_csr(CVMX_ASXX_TX_PRT_EN(oct_eth_info->interface));
        tmp &= ~(1ull << (oct_eth_info->port & 0x3));
        cvmx_write_csr(CVMX_ASXX_TX_PRT_EN(oct_eth_info->interface), tmp);   /* Disable the RGMII TX ports */

        /* Disable MAC filtering */
        gmx_cfg.u64 = cvmx_read_csr(CVMX_GMXX_PRTX_CFG(index, oct_eth_info->interface));
        cvmx_write_csr(CVMX_GMXX_PRTX_CFG(index, oct_eth_info->interface), gmx_cfg.u64 & ~1ull);
        cvmx_write_csr(CVMX_GMXX_RXX_ADR_CTL(index, oct_eth_info->interface), 1);
        cvmx_write_csr(CVMX_GMXX_RXX_ADR_CAM_EN(index, oct_eth_info->interface), 0);
        cvmx_write_csr(CVMX_GMXX_PRTX_CFG(index, oct_eth_info->interface), gmx_cfg.u64);
    }

}

/*********************************************************
Only handle built-in RGMII/SGMII/XAUI/GMII ports here
*********************************************/
int octeon_eth_initialize (bd_t * bis)
{
    DECLARE_GLOBAL_DATA_PTR;
    struct eth_device *dev;
    octeon_eth_info_t *oct_eth_info;
    int port;
    int interface;
    uint32_t *mac_inc_ptr = (uint32_t *)(&mac_addr[2]);
    int num_ports;
    int num_ints;

    dprintf("octeon_mgmt_eth_initialize called\n");
    if (available_mac_count == -1)
    {
        available_mac_count = gd->mac_desc.count;
        memcpy(mac_addr, (uint8_t *)(gd->mac_desc.mac_addr_base), 6);
    }
    if (getenv("disable_networking"))
    {
        printf("Networking disabled with 'disable_networking' environment variable, skipping RGMII interface\n");
        return 0;
    }
    if (available_mac_count <= 0)
    {
        printf("No available MAC addresses for RGMII interface, skipping\n");
        return 0;
    }
    /* Initialize port state array to invalid values */
    memset(port_state, 0xff, sizeof(port_state));

    if (octeon_is_model(OCTEON_CN38XX_PASS1))
    {
        printf("\n");
        printf("\n");
        printf("WARNING:\n");
        printf("WARNING:OCTEON pass 1 does not support network hardware reset.\n");
        printf("WARNING:Bootloader networking use will cause unstable operation of any other\n");
        printf("WARNING:program or OS using the network hardware.\n");
        printf("WARNING:\n");
        printf("\n");
        printf("\n");
    }

    /* Do board specific init in early_board_init or checkboard if possible. */

    /* NOTE: on 31XX based boards, the GMXX_INF_MODE register must be set appropriately
    ** before this code is run (in checkboard for instance).  The hardware is configured based
    ** on the settings of GMXX_INF_MODE. */

    /* We only care about the first two (real packet interfaces)
    ** This will ignore PCI, loop, etc interfaces. */
    num_ints = cvmx_helper_get_number_of_interfaces();
    dprintf("Num interfaces: %d\n", num_ints);
    if (num_ints > 2)
        num_ints = 2;

    /* Check to see what interface and ports we should use */
    for (interface = 0; interface < num_ints; interface++)
    {
        if (CVMX_HELPER_INTERFACE_MODE_RGMII == cvmx_helper_interface_get_mode(interface)
            || CVMX_HELPER_INTERFACE_MODE_GMII == cvmx_helper_interface_get_mode(interface)
            || CVMX_HELPER_INTERFACE_MODE_SGMII == cvmx_helper_interface_get_mode(interface)
            || CVMX_HELPER_INTERFACE_MODE_XAUI == cvmx_helper_interface_get_mode(interface))
        {
            cvmx_helper_interface_probe(interface);
            num_ports = cvmx_helper_ports_on_interface(interface);
            dprintf("Interface %d is %s, ports: %d\n", interface, cvmx_helper_interface_mode_to_string(cvmx_helper_interface_get_mode(interface)), num_ports);
            /* RGMII or SGMII */
            int index = 0;
            for (index = 0, port = 16 * interface; (port < num_ports + 16 * interface) && available_mac_count-- > 0; port++, index++)
            {
                dev = (struct eth_device *) malloc(sizeof(*dev));

                /* Save pointer for port 16 or 0 to use in IPD reset workaround */
                if (port == 16 || port == 0)
                    loopback_dev = dev;

                oct_eth_info = (octeon_eth_info_t *) malloc(sizeof(octeon_eth_info_t));

                oct_eth_info->port = port;
                oct_eth_info->queue = cvmx_pko_get_base_queue(oct_eth_info->port);
                oct_eth_info->interface = interface;
                oct_eth_info->initted_flag = 0;

                dprintf("Setting up port: %d, queue: %d, int: %d\n", oct_eth_info->port, oct_eth_info->queue, oct_eth_info->interface);
                sprintf (dev->name, "octeth%d", card_number);
                card_number++;

                dev->priv = (void *)oct_eth_info;
                dev->iobase = 0;
                dev->init = octeon_ebt3000_rgmii_init;
                dev->halt = octeon_eth_halt;
                dev->send = octeon_eth_send;
                dev->recv = octeon_eth_recv;
                memcpy(dev->enetaddr, mac_addr, 6);
                (*mac_inc_ptr)++;  /* increment MAC address */

                eth_register (dev);

            }
#if 0
            /* Check to see if we have any outstanding doorbells in any queues we might use.
            ** Warn if we find any, as that means the hardware is not in the expected (reset) state.
            ** The PKO has not been enabled at this point, so this may not be a good place to check this.
            */
            for (port = 16 * interface; (port < num_ports + 16 * interface); port++)
            {
                uint32_t dbell = cvmx_cmd_queue_length(CVMX_CMD_QUEUE_PKO(cvmx_pko_get_base_queue(port)));
                if (dbell)
                {
                    printf("WARNING: queue %d has a non-zero doorbell count of %d\n", cvmx_pko_get_base_queue(port), dbell);
                }
            }
#endif
        }
    }


    return card_number;
}



#ifdef OCTEON_SPI4000_ENET
int octeon_spi4000_initialize(bd_t * bis)
{
    DECLARE_GLOBAL_DATA_PTR;
    struct eth_device *dev;
    octeon_eth_info_t *oct_eth_info;
    int port;
    int interface = 0;
    uint32_t *mac_inc_ptr = (uint32_t *)(&mac_addr[2]);
    cvmx_spi_callbacks_t spi_callbacks_struct;

    if (available_mac_count == -1)
    {
        available_mac_count = gd->mac_desc.count;
        memcpy(mac_addr, (void *)(gd->mac_desc.mac_addr_base), 6);
    }
    if (getenv("disable_networking"))
    {
        printf("Networking disabled with 'disable_networking' environment variable, skipping SPI interface\n");
        return 0;
    }

    /* Re-setup callbacks here, as the initialization done at compile time points
    ** to the link addresses in flash.
    */
    cvmx_spi_get_callbacks(&spi_callbacks_struct);
    spi_callbacks_struct.reset_cb            = cvmx_spi_reset_cb;
    spi_callbacks_struct.calendar_setup_cb   = cvmx_spi_calendar_setup_cb;
    spi_callbacks_struct.clock_detect_cb     = cvmx_spi_clock_detect_cb;
    spi_callbacks_struct.training_cb         = cvmx_spi_training_cb;
    spi_callbacks_struct.calendar_sync_cb    = cvmx_spi_calendar_sync_cb;
    spi_callbacks_struct.interface_up_cb     = cvmx_spi_interface_up_cb;
    cvmx_spi_set_callbacks(&spi_callbacks_struct);

    if (!cvmx_spi4000_is_present(interface))
        return 0;
    if (available_mac_count <= 0)
    {
        printf("No available MAC addresses for SPI interface, skipping\n");
        return 0;
    }

    for (port = 0; port < 10 && available_mac_count-- > 0; port++)
    {
        dev = (struct eth_device *) malloc(sizeof(*dev));
        oct_eth_info = (octeon_eth_info_t *) malloc(sizeof(octeon_eth_info_t));

        oct_eth_info->port = port;
        oct_eth_info->queue = cvmx_pko_get_base_queue(oct_eth_info->port);
        oct_eth_info->interface = interface;
        oct_eth_info->initted_flag = 0;

        dprintf("Setting up SPI4000 port: %d, queue: %d, int: %d\n", oct_eth_info->port, oct_eth_info->queue, oct_eth_info->interface);
        sprintf (dev->name, "octspi%d", port);
        card_number++;

        dev->priv = (void *)oct_eth_info;
        dev->iobase = 0;
        dev->init = octeon_spi4000_init;
        dev->halt = octeon_eth_halt;
        dev->send = octeon_eth_send;
        dev->recv = octeon_eth_recv;
        memcpy(dev->enetaddr, mac_addr, 6);
        (*mac_inc_ptr)++;  /* increment MAC address */

        eth_register (dev);
    }
    return card_number;
}
#endif  /* OCTEON_SPI4000_ENET */


#endif   /* OCTEON_INTERNAL_ENET */


#if defined(CONFIG_MII) || (CONFIG_COMMANDS & CFG_CMD_MII)

/**
 * Perform an MII read. Called by the generic MII routines
 *
 * @param dev      Device to perform read for
 * @param phy_id   The MII phy id
 * @param location Register location to read
 * @return Result from the read or zero on failure
 */

int  miiphy_read(unsigned char addr, unsigned char reg, unsigned short * value)
{

    int bus_id = 0;
    int retval = cvmx_mdio_read(bus_id, addr, reg);
    if (retval < 0)
        return -1;
    *value = (unsigned short)retval;
    return 0;
}

/* Wrapper to make non-error checking code simpler */
int miiphy_read_wrapper(unsigned char  addr, unsigned char  reg)
{
    unsigned short value;

    if (miiphy_read(addr, reg, &value) < 0)
    {
        printf("ERROR: miiphy_read_wrapper(0x%x, 0x%x) timed out.\n", addr, reg);
        return (-1);
    }

    return(value);
}



int miiphy_write(unsigned char  addr,
		 unsigned char  reg,
		 unsigned short value)
{
    int bus_id = 0;
    if (cvmx_mdio_write(bus_id, addr, reg, value) < 0)
        return(-1);
    else
        return(0);
}

#endif /* MII */



#if defined(OCTEON_MGMT_ENET) && (CONFIG_COMMANDS & CFG_CMD_NET)


int octeon_mgmt_eth_init(struct eth_device *dev, bd_t * bis)         /* Initialize the device	*/
{
    octeon_eth_info_t* priv = (octeon_eth_info_t*)dev->priv;
    uint64_t mac = 0x0;
    dprintf("octeon_mgmt_eth_init(), dev_ptr: %p\n", dev);

    if (priv->initted_flag)
        return 1;

    cvmx_mgmt_port_initialize(priv->port);
    memcpy(((char *)&mac) + 2, dev->enetaddr,6);
    cvmx_mgmt_port_set_mac(priv->port, mac);

    priv->enabled = 0;


    priv->initted_flag = 1;
    return(1);
}
int octeon_mgmt_eth_send(struct eth_device *dev, volatile void *packet, int length)	   /* Send a packet	*/
{
    int retval;
    octeon_eth_info_t* priv = (octeon_eth_info_t*)dev->priv;

    if (!priv->enabled)
    {
       priv->enabled = 1;
       cvmx_mgmt_port_enable(priv->port);
    }
    retval = cvmx_mgmt_port_send(priv->port, length, (void *)packet);
    return retval;

}
#define MGMT_BUF_SIZE   1700
int octeon_mgmt_eth_recv(struct eth_device *dev)         /* Check for received packets	*/
{
    uchar buffer[MGMT_BUF_SIZE];
    octeon_eth_info_t* priv = (octeon_eth_info_t*)dev->priv;


    if (!priv->enabled)
    {
       priv->enabled = 1;
       cvmx_mgmt_port_enable(priv->port);
    }
    int length = cvmx_mgmt_port_receive(priv->port, MGMT_BUF_SIZE, buffer);

    if (length > 0)
    {
#if defined(DEBUG) && 0
    /* Dump out packet contents */
    {
	int i, j;
	unsigned char *up = buffer;

	for (i = 0; (i + 16) < length; i += 16)
	{
	    printf("%04x ", i);
	    for (j = 0; j < 16; ++j)
	    {
		printf("%02x ", up[i+j]);
	    }
	    printf("    ");
	    for (j = 0; j < 16; ++j)
	    {
		printf("%c", ((up[i+j] >= ' ') && (up[i+j] <= '~')) ? up[i+j] : '.');
	    }
	    printf("\n");
	}
	printf("%04x ", i);
	for (j = 0; i+j < length; ++j)
	{
	    printf("%02x ", up[i+j]);
	}
	for (; j < 16; ++j)
	{
	    printf("   ");
	}
	printf("    ");
	for (j = 0; i+j < length; ++j)
	{
	    printf("%c", ((up[i+j] >= ' ') && (up[i+j] <= '~')) ? up[i+j] : '.');
	}
	printf("\n");
    }
#endif
        NetReceive (buffer, length);
    }
    else if (length < 0)
    {
        printf("MGMT port rx error: %d\n", length);
    }
    return(length);
}

void octeon_mgmt_eth_halt(struct eth_device *dev)			/* stop SCC			*/
{
    octeon_eth_info_t* priv = (octeon_eth_info_t*)dev->priv;
    priv->enabled = 0;
    cvmx_mgmt_port_disable(priv->port);
}
int octeon_mgmt_eth_initialize (bd_t * bis)
{
    DECLARE_GLOBAL_DATA_PTR;
    struct eth_device *dev;
    octeon_eth_info_t *oct_eth_info;
    uint32_t *mac_inc_ptr = (uint32_t *)(&mac_addr[2]);
    int mgmt_port_count = 1;
    int cur_port;

    if (OCTEON_IS_MODEL(OCTEON_CN52XX))
        mgmt_port_count = 2;


    dprintf("octeon_mgmt_eth_initialize called\n");
    if (available_mac_count == -1)
    {
        available_mac_count = gd->mac_desc.count;
        memcpy(mac_addr, (uint8_t *)(gd->mac_desc.mac_addr_base), 6);
    }
    if (getenv("disable_networking"))
    {
        printf("Networking disabled with 'disable_networking' environment variable, skipping RGMII interface\n");
        return 0;
    }

    if (available_mac_count <= 0)
    {
        printf("No available MAC addresses for Management interface(s), skipping\n");
        return 0;
    }


    for (cur_port = 0; (cur_port < mgmt_port_count) && available_mac_count-- > 0; cur_port++)
    {
        /* Initialize port state array to invalid values */
        memset(port_state, 0xff, sizeof(port_state));

        dev = (struct eth_device *) malloc(sizeof(*dev));

        oct_eth_info = (octeon_eth_info_t *) malloc(sizeof(octeon_eth_info_t));

        oct_eth_info->port = cur_port;
        /* These don't apply to the management ports */
        oct_eth_info->queue = ~0;
        oct_eth_info->interface = ~0;
        oct_eth_info->initted_flag = 0;

        dprintf("Setting up mgmt eth port\n");
        sprintf (dev->name, "octmgmt%d", cur_port);

        dev->priv = (void *)oct_eth_info;
        dev->iobase = 0;
        dev->init = octeon_mgmt_eth_init;
        dev->halt = octeon_mgmt_eth_halt;
        dev->send = octeon_mgmt_eth_send;
        dev->recv = octeon_mgmt_eth_recv;
        memcpy(dev->enetaddr, mac_addr, 6);

        (*mac_inc_ptr)++;  /* increment MAC address */

        eth_register (dev);
    }

    return card_number;

}

#endif
