/*
* Manufacturing infomration for SMB Security & Wireless products
*
*	Author : Satish Nandi
*
*	This header defines common structures which covers all the 
*	products based on current information & roadmap
*	This data structure is designed to use single common hardware with 
*	different manufacturing time populated features.
*
*	This is also indented to use single firmware/binary that can run for 
*	multiple products of similar functionality.
*
*	Version 0.0 -- June 21, 2007
*		Base structure definition
*	
*		Common hardware shall support
*			- 2 Wan ports with VLAN capable 
*			- Upto 8 LAN port with VLAN capable
*			- RAM - 16MB to 1024MB
*			- Flash - 2MB to 256MB
*			- 2 Mini PCI
*			- Serial port - Console Only, Full modem & both
*			- Hardware version 
*
*
*
*
*
*/

#include <config.h>

#define BYTE unsigned char 
#define SHORT unsigned short
#define WORD unsigned short

/*
*	Product ID numbers
*/
#define PATRIC_LO	0 	/* Use the configuration in board Info */

/* CFG_FLASH_BASE = 0xBFC0_0000 - CFG_FLASH_SIZE*/

//#define	MANUFACTURING_FLASH_START	0xbec00000+0xc0000
#define	MANUFACTURING_FLASH_START	CFG_MD_ADDR
#define	MANUFACTURING_RAM_START		0xe0000


/*
*
*	Port Information : 
*	High bytex
*		7-4 bits --- Number of MAC for this port
*		3 bit    --- 1 for POE 0 for NON-POE
*		2-0 bits --- Reserved 
*
*	Low Byte
*		7-4 bits -- Reserved.
*		3 bit --- VLAN 1 enable 0 not allowed.
*		2-0 bits --- Speeds for this port
*			000 --- Port disabled/Not presetn
*			001 --- 10 Only
*			010 --- 100 Only
*			100 --- 1000 Only
*			111 --- 10/100/1000 & other combination as used.
*/

typedef struct boardinfo {
	SHORT	cpuinfo;	/* Required ???? */
	SHORT	freq;		/* In MHz */
	SHORT  	ram;		/* RAM in MBs 0 - 1024 */
	SHORT	flash;		/* Flash in MBs */
	SHORT 	poe_pwr;	/* In watts */
	SHORT	portw1;		
	SHORT	portw2;	
	SHORT   portw3;
        SHORT   portw4;
	SHORT	portlan[8];	
	SHORT 	minipci[2];	/* Bit 15 Card En/Dis -- 14-0 Card Type */
	SHORT	serialport;	/* None - 0 Con only-1,Modem only-2, both 3 */
} boardinfo_t;



/*
* Field of manufacturing data
*	Product ID --- Provides the ID of the product.
*	HWVer	   --- ASCII in WW.XX.YY.ZZ 
*	reginfo    --- Regional information 0 WW, 1 NA, 2 ..... 
*	numofimages -- Number of images for the product 
*	currimage  --- Questionable field ? Pointer to current active image 
*	basemac ------ Base Mac address for the product is stored in ASCII 
*	maccnt  ------ Total number of numbers
*	serno      --- Stored in ASCII with NULL termination 
*	boardinfo ---- Detailed board information 
*
*/


typedef struct manuinfo {
//	Option - 1
//	WORD	 ProductID;	/* Product ID in numeric */
//					Perfer to have numeric with defines:
//	Option - 2
	BYTE 	ProductID[32];	/* Product ID in ASCII (alpha-numeric) */
	BYTE 	HWVer[12];	/* Stored in ASCII in WW.XX.YY.ZZ */
	SHORT 	reginfo;	/* Regional information */
	BYTE	numofimages;	/* Number of images for the product */
	BYTE	currimage;	/* Questionable field ?? \
				Pointer to current active image */
	unsigned int kernelchecksum[2]; /* contains the checksum of two kernel images */
	long long kernelsize[2]; /* contains the size of two kernel images */
	unsigned char version[2][16]; /* firmware version of two images*/
	unsigned int image_status[2];
	BYTE 	basemac[12];	/* Mac digit is stored in ASCII */ 
	WORD 	maccnt[4]; 
	BYTE 	serno[32];	/* Stored in ASCII with NULL termination */ 
	/* 
	*	Align it to cache line 32 byte 
	*/
	boardinfo_t boardinfo;
	BYTE	reserverd[256];
} ManuInfo_t;
