/*
 * (C) Copyright 2003
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 * Copyright 2004,2005 Cavium Networks
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <malloc.h>
#include <devices.h>
#include <net.h>
#include <environment.h>
#include "octeon_boot.h"
#include <lib_octeon.h>
#include <octeon_eeprom_types.h>
#include <lib_octeon_shared.h>
#include "octeon-pci-console.h"
#include "cvmx-bootmem.h"
#include "octeon-model.h"
#include <octeon_boot.h>
#include <macinfo.h>


#if (!defined(ENV_IS_EMBEDDED) ||  defined(CFG_ENV_IS_IN_NVRAM))
#define	TOTAL_MALLOC_LEN	(CFG_MALLOC_LEN + CFG_ENV_SIZE)
#else
#define	TOTAL_MALLOC_LEN	CFG_MALLOC_LEN
#endif


#define DRAM_LATE_ZERO_OFFSET  0x100000ull
// #define DEBUG


/* Global variable use to locate pci console descriptor */
uint64_t pci_cons_desc_addr;

extern int timer_init(void);

extern int incaip_set_cpuclk(void);

extern ulong uboot_end_data;
extern ulong uboot_end;

ulong monitor_flash_len;
#define SET_K0(val)                 asm volatile ("add $26, $0, %[rt]" : : [rt] "d" (val):"memory")

extern const char version_string[];

static char *failed = "*** failed ***\n";

/*
 * Begin and End of memory area for malloc(), and current "brk"
 */
static ulong mem_malloc_start;
static ulong mem_malloc_end;
static ulong mem_malloc_brk;

/* Used for error checking on load commands */
ulong load_reserved_addr = 0; /* Base address of reserved memory for loading */
ulong load_reserved_size = 0; /* Size of reserved memory for loading */

/*
 * The Malloc area is immediately below the monitor copy in DRAM
 */
static void mem_malloc_init (void)
{
	DECLARE_GLOBAL_DATA_PTR;

	ulong dest_addr = CFG_MONITOR_BASE + gd->reloc_off;

	mem_malloc_end = dest_addr;
	mem_malloc_start = dest_addr - TOTAL_MALLOC_LEN;
	mem_malloc_brk = mem_malloc_start;

#if !CONFIG_OCTEON_SIM
	memset ((void *) mem_malloc_start,
		0,
		mem_malloc_end - mem_malloc_start);
#endif
}

void *sbrk (ptrdiff_t increment)
{
	ulong old = mem_malloc_brk;
	ulong new = old + increment;

	if ((new < mem_malloc_start) || (new > mem_malloc_end)) {
		return (NULL);
	}
	mem_malloc_brk = new;
	return ((void *) old);
}


static int init_func_ram (void)
{
    DECLARE_GLOBAL_DATA_PTR;

    puts ("DRAM:  ");

#if CONFIG_OCTEON_SIM
    if ((gd->ram_size = (uint64_t)(*((uint32_t *)0x9ffffff8) * 1024 * 1024)) > 0) {
        print_size (gd->ram_size, "\n");
        return (0);
    }
#else
    if ((gd->ram_size) > 0) {
        print_size (gd->ram_size, "\n");
        return (0);
    }
#endif

    puts (failed);
    return (1);
}

static int display_banner(void)
{
	//turn on TEST LED -- by Joe
	octeon_gpio_cfg_output(13);
	octeon_gpio_clr(13);

	printf ("\n\n%s\n\n", version_string);
	return (0);
}

static void display_flash_config(ulong size)
{
	puts ("Flash: ");
	print_size (size, "\n");
}

#if !CONFIG_OCTEON_SIM
static const dimm_odt_config_t dimm_odt_cn31xx_1rank_config_table[] = {
    CN31XX_DRAM_ODT_1RANK_CONFIGURATION
};
static const dimm_odt_config_t dimm_odt_cn31xx_2rank_config_table[] = {
    CN31XX_DRAM_ODT_2RANK_CONFIGURATION
};
static const dimm_odt_config_t dimm_odt_cn38xx_1rank_config_table[] = {
    CN38XX_DRAM_ODT_1RANK_CONFIGURATION
};
static const dimm_odt_config_t dimm_odt_cn38xx_2rank_config_table[] = {
    CN38XX_DRAM_ODT_2RANK_CONFIGURATION
};
static const dimm_odt_config_t dimm_odt_cn58xx_1rank_config_table[] = {
    CN58XX_DRAM_ODT_1RANK_CONFIGURATION
};
static const dimm_odt_config_t dimm_odt_cn58xx_2rank_config_table[] = {
    CN58XX_DRAM_ODT_2RANK_CONFIGURATION
};
static const dimm_odt_config_t dimm_odt_cn50xx_1rank_config_table[] = {
    CN50XX_DRAM_ODT_1RANK_CONFIGURATION
};
static const dimm_odt_config_t dimm_odt_cn50xx_2rank_config_table[] = {
    CN50XX_DRAM_ODT_2RANK_CONFIGURATION
};
static const dimm_odt_config_t dimm_odt_cn56xx_1rank_config_table[] = {
    CN56XX_DRAM_ODT_1RANK_CONFIGURATION
};
static const dimm_odt_config_t dimm_odt_cn56xx_2rank_config_table[] = {
    CN56XX_DRAM_ODT_2RANK_CONFIGURATION
};
static const dimm_odt_config_t dimm_odt_cn52xx_1rank_config_table[] = {
    CN52XX_DRAM_ODT_1RANK_CONFIGURATION
};
static const dimm_odt_config_t dimm_odt_cn52xx_2rank_config_table[] = {
    CN52XX_DRAM_ODT_2RANK_CONFIGURATION
};


static int init_dram(void)
{
    DECLARE_GLOBAL_DATA_PTR;
    int memsize;
    int retval;
    char *eptr;
    ddr_configuration_t ddr_configuration;

#if !(CONFIG_RAM_RESIDENT)
    /* Init the L2C, must be done before DRAM access so that we know L2 is empty */
    eptr = getenv("disable_l2_index_aliasing");
    if (eptr)
    {
        printf("L2 index aliasing disabled.\n");
    }
    else
    {
        /* Enable L2C index aliasing */
        uint64_t tmp = cvmx_read_csr(CVMX_L2C_CFG);
        cvmx_write_csr(CVMX_L2C_CFG,  tmp | (1ull << 7));
    }
#endif

    /* Check to see if we should limit the number of L2 ways. */
    eptr = getenv("limit_l2_ways");
    if (eptr)
    {
        uint32_t valid_mask = 0xff;
        int ways_max = 7;
        if (!octeon_is_model(OCTEON_CN38XX))
        {
            valid_mask = 0xf;
            ways_max = 3;
        }
        
        int ways = simple_strtoul(eptr, NULL, 10);
        if (ways >= 1 && ways <= ways_max)
        {
            printf("Limiting L2 to %d ways\n", ways);
            uint64_t mask = (0xff << ways) & valid_mask;

            if (!octeon_is_model(OCTEON_CN30XX))
            {
                mask |= mask << 8;
                if (octeon_is_model(OCTEON_CN38XX))
                {
                    mask |= mask << 8;
                    mask |= mask << 8;
                }
            }
            cvmx_write_csr(CVMX_L2C_SPAR0, mask);
            if (octeon_is_model(OCTEON_CN38XX))
            {
                cvmx_write_csr(CVMX_L2C_SPAR1, mask);
                cvmx_write_csr(CVMX_L2C_SPAR2, mask);
                cvmx_write_csr(CVMX_L2C_SPAR3, mask);
            }
            cvmx_write_csr(CVMX_L2C_SPAR4, mask & valid_mask);
        }
        else
            printf("Invalid limit_l2_ways value: %d, must be between 1 and 7\n", ways);
    }


    uint32_t ddr_hertz = gd->ddr_clock_mhz * 1000000;
    uint32_t cpu_hertz = gd->cpu_clock_mhz * 1000000;
    uint32_t ddr_ref_hertz = gd->ddr_ref_hertz;

    char *s;

    if ((s = getenv("ddr_clock_hertz")) != NULL) {
        ddr_hertz = simple_strtoul(s, NULL, 0);
        gd->ddr_clock_mhz = ddr_hertz / 1000000;
        printf("Parameter found in environment. ddr_clock_hertz = %d\n", ddr_hertz);
    }

    if ((s = getenv("ddr_ref_hertz")) != NULL) {
        ddr_ref_hertz = simple_strtoul(s, NULL, 0);
        gd->ddr_ref_hertz = ddr_ref_hertz;
        printf("Parameter found in environment. ddr_ref_hertz = %d\n", ddr_ref_hertz);
    }

    uint32_t ddr_interface_mask = 0;

    dimm_config_t dimm_config_table[] = 
        {
#ifdef DRAM_SOCKET_CONFIGURATION
            DRAM_SOCKET_CONFIGURATION,
#endif /* DRAM_SOCKET_CONFIGURATION */
            {
                0,
                0
            }
        };

    /* Check for lower DIMM socket populated */
    if (validate_dimm(dimm_config_table[0].lower_spd_addr, 0, 0)) {
        ddr_interface_mask = 1;
    }

#ifdef DRAM_SOCKET_CONFIGURATION_INTERFACE1
    dimm_config_t dimm_config_table_interface1[] = 
        {
            DRAM_SOCKET_CONFIGURATION_INTERFACE1,
            {
                0,
                0
            }
        };

    if (validate_dimm(dimm_config_table_interface1[0].lower_spd_addr, 0, 0)) {
        ddr_interface_mask |= 1<<1;
    }
#endif /* DRAM_SOCKET_CONFIGURATION_INTERFACE1 */


    /* Except for Octeon pass1 we can count DDR clocks, and we use
    ** this to correct the DDR clock that we are passed. For Octeon
    ** pass1 the measured value will always be returned as 0. */
    uint32_t calc_ddr_hertz = measure_octeon_ddr_clock(cvmx_get_proc_id(), cpu_hertz, ddr_hertz, ddr_ref_hertz, 0, ddr_interface_mask);

    if (ddr_interface_mask & 2) {
        calc_ddr_hertz = measure_octeon_ddr_clock(cvmx_get_proc_id(), cpu_hertz, ddr_hertz, ddr_ref_hertz, 1, ddr_interface_mask);
    }

#ifdef DEBUG
    int calc_ddr_megahertz = calc_ddr_hertz / 1000000;
    printf("Measured DDR clock %d.%02d MHz\n", calc_ddr_megahertz, (calc_ddr_hertz - calc_ddr_megahertz * 1000000) / CALC_SCALER);
#endif

    if (calc_ddr_hertz != 0)
    {
        if (!gd->ddr_clock_mhz)
        {
            /* If ddr_clock not set, use measured clock and don't warn */
            gd->ddr_clock_mhz = calc_ddr_hertz/1000000;
        }
        else if ((calc_ddr_hertz > ddr_hertz + 3000000) || (calc_ddr_hertz <  ddr_hertz - 3000000))
        {
            printf("\nWARNING:\n");
            printf("WARNING: Measured DDR clock mismatch! expected: %d MHz, measured: %d MHz, cpu clock: %d MHz\n",
                   ddr_hertz/1000000, calc_ddr_hertz/1000000, cpu_hertz/1000000);
#if !CONFIG_RAM_RESIDENT
            printf("WARNING: Using measured clock for configuration.\n");
#endif
            printf("WARNING:\n\n");
            gd->ddr_clock_mhz = calc_ddr_hertz/1000000;
        }
    }


    /* Enable L2 ECC */
    cvmx_write_csr(CVMX_L2D_ERR, 1);
    cvmx_write_csr(CVMX_L2T_ERR, 1);


    if (gd->flags & GD_FLG_CLOCK_DESC_MISSING)
    {
        printf("Warning: Clock descriptor tuple not found in eeprom, using defaults\n");
        if (gd->board_desc.rev_major == 1)
        {
            printf("\n\nWARNING: memory configured for %d mhz clock.  \nIf this is not the actual memory clock\n", gd->ddr_clock_mhz);
            printf("poor performance and memory instability may result.  \nThe memory speed must be specified in the board eeprom\n\n\n");
        }
    }
    if (gd->flags & GD_FLG_BOARD_DESC_MISSING)
    {
        printf("Warning: Board descriptor tuple not found in eeprom, using defaults\n");
    }


    printf("%s board revision major:%d, minor:%d, serial #: %s\n",
           cvmx_board_type_to_string(gd->board_desc.board_type),
           gd->board_desc.rev_major, gd->board_desc.rev_minor,
           gd->board_desc.serial_str);

    {
        char buffer[32];
        octeon_model_get_string_buffer(cvmx_get_proc_id(), buffer);
        printf("OCTEON %s, Core clock: %d MHz, DDR clock: %d MHz (%d Mhz data rate)\n",
               buffer,
               gd->cpu_clock_mhz,
               gd->ddr_clock_mhz, gd->ddr_clock_mhz *2);
    }




    /* Some boards need different ddr delay based on revision */
#if CONFIG_OCTEON_EBH3100
    if (gd->board_desc.rev_major == 1) {
        ddr_configuration.registered.ddr_board_delay = DDR_BOARD_DELAY_REV_1;
        ddr_configuration.unbuffered.ddr_board_delay = DDR_BOARD_DELAY_REV_1; /* Untested */
    }
    else {
        ddr_configuration.registered.ddr_board_delay = DDR_BOARD_DELAY_REV_2;
        ddr_configuration.unbuffered.ddr_board_delay = DDR_BOARD_DELAY_REV_2; /* Untested */
    }
#else
#ifdef DDR_BOARD_DELAY
    /* Board delay can vary based on configuration.  A default
       configuration can be described by DDR_BOARD_DELAY. */
    ddr_configuration.registered.ddr_board_delay = DDR_BOARD_DELAY;
    ddr_configuration.unbuffered.ddr_board_delay = DDR_BOARD_DELAY;
#endif
#ifdef DDR_BOARD_DELAY_UNBUFFERED
    /* Board delay can vary based on configuration.  A default
       configuration can be described by DDR_BOARD_DELAY. */
    ddr_configuration.unbuffered.ddr_board_delay = DDR_BOARD_DELAY_UNBUFFERED;
#endif
#endif
    uint32_t cpu_id = cvmx_get_proc_id();

    /* Check for special case of mismarked 3005 samples, and adjust cpuid */
    if (cpu_id == OCTEON_CN3010_PASS1 && (cvmx_read_csr(0x80011800800007B8ull) & (1ull << 34)))
        cpu_id |= 0x10;

    if (octeon_is_model(OCTEON_CN31XX) || octeon_is_model(OCTEON_CN30XX))
    {
        ddr_configuration.odt_1rank_config = (dimm_odt_config_t *)dimm_odt_cn31xx_1rank_config_table;
        ddr_configuration.odt_2rank_config = (dimm_odt_config_t *)dimm_odt_cn31xx_2rank_config_table;
    }
    else if (octeon_is_model(OCTEON_CN58XX))
    {
        ddr_configuration.odt_1rank_config = (dimm_odt_config_t *)dimm_odt_cn58xx_1rank_config_table;
        ddr_configuration.odt_2rank_config = (dimm_odt_config_t *)dimm_odt_cn58xx_2rank_config_table;
    }
    else if (octeon_is_model(OCTEON_CN50XX))
    {
        ddr_configuration.odt_1rank_config = (dimm_odt_config_t *)dimm_odt_cn50xx_1rank_config_table;
        ddr_configuration.odt_2rank_config = (dimm_odt_config_t *)dimm_odt_cn50xx_2rank_config_table;
    }
    else if (octeon_is_model(OCTEON_CN56XX))
    {
        ddr_configuration.odt_1rank_config = (dimm_odt_config_t *)dimm_odt_cn56xx_1rank_config_table;
        ddr_configuration.odt_2rank_config = (dimm_odt_config_t *)dimm_odt_cn56xx_2rank_config_table;
    }
    else if (octeon_is_model(OCTEON_CN52XX))
    {
        ddr_configuration.odt_1rank_config = (dimm_odt_config_t *)dimm_odt_cn52xx_1rank_config_table;
        ddr_configuration.odt_2rank_config = (dimm_odt_config_t *)dimm_odt_cn52xx_2rank_config_table;
    }
    else
    {
        ddr_configuration.odt_1rank_config = (dimm_odt_config_t *)dimm_odt_cn38xx_1rank_config_table;
        ddr_configuration.odt_2rank_config = (dimm_odt_config_t *)dimm_odt_cn38xx_2rank_config_table;
    }

#if (CONFIG_OCTEON_EBH5600 || CONFIG_OCTEON_EBH5601)
    if (OCTEON_IS_MODEL(OCTEON_CN56XX)) {
        ddr_configuration.registered.ddr_board_delay = OCTEON_CN56XX_EBH5600_DDR_BOARD_DELAY;
        ddr_configuration.unbuffered.ddr_board_delay = OCTEON_CN56XX_EBH5600_UNB_DDR_BOARD_DELAY;
    }
#endif

#if (CONFIG_OCTEON_EBH5200)
    if (OCTEON_IS_MODEL(OCTEON_CN52XX)) {
        ddr_configuration.registered.ddr_board_delay = OCTEON_CN52XX_EBH5200_DDR_BOARD_DELAY;
        ddr_configuration.unbuffered.ddr_board_delay = OCTEON_CN52XX_EBH5200_UNB_DDR_BOARD_DELAY;
    }
#endif

#if (CONFIG_OCTEON_EBT5800)
    if (OCTEON_IS_MODEL(OCTEON_CN58XX)) {
        ddr_configuration.registered.ddr_board_delay = OCTEON_CN58XX_EBT5800_DDR_BOARD_DELAY;
        ddr_configuration.unbuffered.ddr_board_delay = OCTEON_CN58XX_EBT5800_UNB_DDR_BOARD_DELAY;
    }
    if (OCTEON_IS_MODEL(OCTEON_CN38XX)){
        ddr_configuration.registered.ddr_board_delay = OCTEON_CN38XX_EBT5800_DDR_BOARD_DELAY;
        ddr_configuration.unbuffered.ddr_board_delay = OCTEON_CN38XX_EBT5800_UNB_DDR_BOARD_DELAY;
    }
#endif

#if (CONFIG_OCTEON_CN3010_EVB_HS5)
    if (OCTEON_IS_MODEL(OCTEON_CN50XX)) {
        ddr_configuration.registered.ddr_board_delay = OCTEON_CN50XX_CN3010_EVB_HS5_DDR_BOARD_DELAY;
        ddr_configuration.unbuffered.ddr_board_delay = OCTEON_CN50XX_CN3010_EVB_HS5_UNB_DDR_BOARD_DELAY;
    }
#endif

#if (CONFIG_OCTEON_EBT3000)
    ddr_configuration.registered.ddr_board_delay = OCTEON_EBT3000_DDR_BOARD_DELAY;
    ddr_configuration.unbuffered.ddr_board_delay = OCTEON_EBT3000_UNB_DDR_BOARD_DELAY;
#endif

#if (CONFIG_OCTEON_NAC38)
    if (OCTEON_IS_MODEL(OCTEON_CN58XX)) {
        ddr_configuration.registered.ddr_board_delay = OCTEON_CN58XX_NAC38_DDR_BOARD_DELAY;
        ddr_configuration.unbuffered.ddr_board_delay = OCTEON_CN58XX_NAC38_UNB_DDR_BOARD_DELAY;
    }
    if (OCTEON_IS_MODEL(OCTEON_CN38XX)){
        ddr_configuration.registered.ddr_board_delay = OCTEON_CN38XX_NAC38_DDR_BOARD_DELAY;
        ddr_configuration.unbuffered.ddr_board_delay = OCTEON_CN38XX_NAC38_UNB_DDR_BOARD_DELAY;
    }
#endif



#ifdef LMC_DELAY_CLK  /* Defined per board */
        ddr_configuration.unbuffered.lmc_delay_clk  = LMC_DELAY_CLK;
        ddr_configuration.unbuffered.lmc_delay_cmd  = LMC_DELAY_CMD;
        ddr_configuration.unbuffered.lmc_delay_dq   = LMC_DELAY_DQ;
        ddr_configuration.registered.lmc_delay_clk  = LMC_DELAY_CLK;
        ddr_configuration.registered.lmc_delay_cmd  = LMC_DELAY_CMD;
        ddr_configuration.registered.lmc_delay_dq   = LMC_DELAY_DQ;
#endif

#if (CONFIG_OCTEON_EBH5600 || CONFIG_OCTEON_EBH5601)
        if (OCTEON_IS_MODEL(OCTEON_CN56XX)) {
            ddr_configuration.registered.lmc_delay_clk  = OCTEON_CN56XX_EBH5600_LMC_DELAY_CLK;
            ddr_configuration.registered.lmc_delay_cmd  = OCTEON_CN56XX_EBH5600_LMC_DELAY_CMD;
            ddr_configuration.registered.lmc_delay_dq   = OCTEON_CN56XX_EBH5600_LMC_DELAY_DQ;

            ddr_configuration.unbuffered.lmc_delay_clk  = OCTEON_CN56XX_EBH5600_UNB_LMC_DELAY_CLK;
            ddr_configuration.unbuffered.lmc_delay_cmd  = OCTEON_CN56XX_EBH5600_UNB_LMC_DELAY_CMD;
            ddr_configuration.unbuffered.lmc_delay_dq   = OCTEON_CN56XX_EBH5600_UNB_LMC_DELAY_DQ;
        }
#endif

#if (CONFIG_OCTEON_EBH5200)
        if (OCTEON_IS_MODEL(OCTEON_CN52XX)) {
            ddr_configuration.registered.lmc_delay_clk  = OCTEON_CN52XX_EBH5200_LMC_DELAY_CLK;
            ddr_configuration.registered.lmc_delay_cmd  = OCTEON_CN52XX_EBH5200_LMC_DELAY_CMD;
            ddr_configuration.registered.lmc_delay_dq   = OCTEON_CN52XX_EBH5200_LMC_DELAY_DQ;

            ddr_configuration.unbuffered.lmc_delay_clk  = OCTEON_CN52XX_EBH5200_UNB_LMC_DELAY_CLK;
            ddr_configuration.unbuffered.lmc_delay_cmd  = OCTEON_CN52XX_EBH5200_UNB_LMC_DELAY_CMD;
            ddr_configuration.unbuffered.lmc_delay_dq   = OCTEON_CN52XX_EBH5200_UNB_LMC_DELAY_DQ;
        }
#endif

#if (CONFIG_OCTEON_EBT5800)
        if (OCTEON_IS_MODEL(OCTEON_CN58XX)) {
            ddr_configuration.registered.lmc_delay_clk  = OCTEON_CN58XX_EBT5800_LMC_DELAY_CLK;
            ddr_configuration.registered.lmc_delay_cmd  = OCTEON_CN58XX_EBT5800_LMC_DELAY_CMD;
            ddr_configuration.registered.lmc_delay_dq   = OCTEON_CN58XX_EBT5800_LMC_DELAY_DQ;

            ddr_configuration.unbuffered.lmc_delay_clk  = OCTEON_CN58XX_EBT5800_UNB_LMC_DELAY_CLK;
            ddr_configuration.unbuffered.lmc_delay_cmd  = OCTEON_CN58XX_EBT5800_UNB_LMC_DELAY_CMD;
            ddr_configuration.unbuffered.lmc_delay_dq   = OCTEON_CN58XX_EBT5800_UNB_LMC_DELAY_DQ;
        }
        if (OCTEON_IS_MODEL(OCTEON_CN38XX)) {
            ddr_configuration.registered.lmc_delay_clk  = OCTEON_CN38XX_EBT5800_LMC_DELAY_CLK;
            ddr_configuration.registered.lmc_delay_cmd  = OCTEON_CN38XX_EBT5800_LMC_DELAY_CMD;
            ddr_configuration.registered.lmc_delay_dq   = OCTEON_CN38XX_EBT5800_LMC_DELAY_DQ;

            ddr_configuration.unbuffered.lmc_delay_clk  = OCTEON_CN38XX_EBT5800_UNB_LMC_DELAY_CLK;
            ddr_configuration.unbuffered.lmc_delay_cmd  = OCTEON_CN38XX_EBT5800_UNB_LMC_DELAY_CMD;
            ddr_configuration.unbuffered.lmc_delay_dq   = OCTEON_CN38XX_EBT5800_UNB_LMC_DELAY_DQ;
        }
#endif

#if (CONFIG_OCTEON_CN3010_EVB_HS5)
        if (OCTEON_IS_MODEL(OCTEON_CN50XX)) {
            ddr_configuration.registered.lmc_delay_clk  = OCTEON_CN50XX_CN3010_EVB_HS5_LMC_DELAY_CLK;
            ddr_configuration.registered.lmc_delay_cmd  = OCTEON_CN50XX_CN3010_EVB_HS5_LMC_DELAY_CMD;
            ddr_configuration.registered.lmc_delay_dq   = OCTEON_CN50XX_CN3010_EVB_HS5_LMC_DELAY_DQ;

            ddr_configuration.unbuffered.lmc_delay_clk  = OCTEON_CN50XX_CN3010_EVB_HS5_UNB_LMC_DELAY_CLK;
            ddr_configuration.unbuffered.lmc_delay_cmd  = OCTEON_CN50XX_CN3010_EVB_HS5_UNB_LMC_DELAY_CMD;
            ddr_configuration.unbuffered.lmc_delay_dq   = OCTEON_CN50XX_CN3010_EVB_HS5_UNB_LMC_DELAY_DQ;
        }
#endif

#if (CONFIG_OCTEON_EBT3000)
        ddr_configuration.registered.lmc_delay_clk  = OCTEON_EBT3000_LMC_DELAY_CLK;
        ddr_configuration.registered.lmc_delay_cmd  = OCTEON_EBT3000_LMC_DELAY_CMD;
        ddr_configuration.registered.lmc_delay_dq   = OCTEON_EBT3000_LMC_DELAY_DQ;

        ddr_configuration.unbuffered.lmc_delay_clk  = OCTEON_EBT3000_UNB_LMC_DELAY_CLK;
        ddr_configuration.unbuffered.lmc_delay_cmd  = OCTEON_EBT3000_UNB_LMC_DELAY_CMD;
        ddr_configuration.unbuffered.lmc_delay_dq   = OCTEON_EBT3000_UNB_LMC_DELAY_DQ;
#endif

#if (CONFIG_OCTEON_NAC38)
        if (OCTEON_IS_MODEL(OCTEON_CN58XX)) {
            ddr_configuration.registered.lmc_delay_clk  = OCTEON_CN58XX_NAC38_LMC_DELAY_CLK;
            ddr_configuration.registered.lmc_delay_cmd  = OCTEON_CN58XX_NAC38_LMC_DELAY_CMD;
            ddr_configuration.registered.lmc_delay_dq   = OCTEON_CN58XX_NAC38_LMC_DELAY_DQ;

            ddr_configuration.unbuffered.lmc_delay_clk  = OCTEON_CN58XX_NAC38_UNB_LMC_DELAY_CLK;
            ddr_configuration.unbuffered.lmc_delay_cmd  = OCTEON_CN58XX_NAC38_UNB_LMC_DELAY_CMD;
            ddr_configuration.unbuffered.lmc_delay_dq   = OCTEON_CN58XX_NAC38_UNB_LMC_DELAY_DQ;
        }
        if (OCTEON_IS_MODEL(OCTEON_CN38XX)) {
            ddr_configuration.registered.lmc_delay_clk  = OCTEON_CN38XX_NAC38_LMC_DELAY_CLK;
            ddr_configuration.registered.lmc_delay_cmd  = OCTEON_CN38XX_NAC38_LMC_DELAY_CMD;
            ddr_configuration.registered.lmc_delay_dq   = OCTEON_CN38XX_NAC38_LMC_DELAY_DQ;

            ddr_configuration.unbuffered.lmc_delay_clk  = OCTEON_CN38XX_NAC38_UNB_LMC_DELAY_CLK;
            ddr_configuration.unbuffered.lmc_delay_cmd  = OCTEON_CN38XX_NAC38_UNB_LMC_DELAY_CMD;
            ddr_configuration.unbuffered.lmc_delay_dq   = OCTEON_CN38XX_NAC38_UNB_LMC_DELAY_DQ;
        }
#endif

    ddr_configuration.dimm_config_table = dimm_config_table;

    memsize = 0;
    if (ddr_interface_mask & 1) {
        retval = init_octeon_dram_interface(cpu_id,
                                            &ddr_configuration,
                                            gd->ddr_clock_mhz * 1000000,
                                            gd->cpu_clock_mhz*1000000,
                                            ddr_ref_hertz,
                                            gd->board_desc.board_type,
                                            gd->board_desc.rev_major,
                                            gd->board_desc.rev_minor,
                                            0,
                                            ddr_interface_mask
                                            );
        if (retval > 0)
            memsize = retval;
    }


#ifdef DRAM_SOCKET_CONFIGURATION_INTERFACE1
    if (ddr_interface_mask & 2) {
        ddr_configuration.dimm_config_table = dimm_config_table_interface1;
        retval = init_octeon_dram_interface(cpu_id,
                                              &ddr_configuration,
                                              gd->ddr_clock_mhz * 1000000,
                                              gd->cpu_clock_mhz*1000000,
                                              ddr_ref_hertz,
                                              gd->board_desc.board_type,
                                              gd->board_desc.rev_major,
                                              gd->board_desc.rev_minor,
                                              1,
                                              ddr_interface_mask
                                              );
        if (retval > 0)
            memsize += retval;
    }
#endif

    eptr = getenv("limit_dram_mbytes");
    if (eptr)
    {
        unsigned int mbytes = simple_strtol(eptr, NULL, 10);
        if (mbytes > 0)
        {
            memsize = mbytes;
            printf("Limiting DRAM size to %d MBytes based on limit_dram_mbytes env. variable\n", mbytes);
        }
    }


    if (memsize > 0)
    {
        gd->ram_size = (uint64_t)memsize * 1024 * 1024;

#if !defined(CONFIG_NO_RELOCATION) /* If already running in memory don't zero it. */
    /* Zero the low Megabyte of DRAM used by bootloader.  The rest is zeroed later.
    ** when running from DRAM*/
        octeon_bzero64_pfs(0, DRAM_LATE_ZERO_OFFSET);
#endif /* !CONFIG_NO_RELOCATION */
        return(0);
    }
    else
    {
        return -1;
    }

}
#endif


static int init_baudrate (void)
{
	DECLARE_GLOBAL_DATA_PTR;

	char tmp[64];	/* long enough for environment variables */
	int i = getenv_r ("baudrate", tmp, sizeof (tmp));

	gd->baudrate = (i > 0)
			? (int) simple_strtoul (tmp, NULL, 10)
			: CONFIG_BAUDRATE;

	return (0);
}

#define DO_SIMPLE_DRAM_TEST_FROM_FLASH  0
#if DO_SIMPLE_DRAM_TEST_FROM_FLASH
/* Very simple DRAM test to run from flash for cases when bootloader boot
** can't complete.
*/
static int dram_test(void)
{
    uint32_t *addr;
    uint32_t *start = (void *)0x80000000;
    uint32_t *end = (void *)0x80200000;
    uint32_t val, incr, readback, pattern;
    int error_limit;

    pattern = 0;
    printf("Performing simple DDR test from flash.\n");

    #define DRAM_TEST_ERROR_LIMIT 200

    incr = 1;
    for (;;) {
        error_limit = DRAM_TEST_ERROR_LIMIT;

        printf ("\rPattern %08lX  Writing..."
                "%12s"
                "\b\b\b\b\b\b\b\b\b\b",
                pattern, "");

        for (addr=start,val=pattern; addr<end; addr++) {
                *addr = val;
                val  += incr;
        }

        puts ("Reading...");

        for (addr=start,val=pattern; addr<end; addr++) {
                readback = *addr;
                if (readback != val && error_limit-- > 0) {
                    if (error_limit + 1 == DRAM_TEST_ERROR_LIMIT)
                        puts("\n");
                    printf ("Mem error @ 0x%08X: "
                            "found %08lX, expected %08lX\n",
                            (uint)addr, readback, val);
                }
                val += incr;
        }

        /*
         * Flip the pattern each time to make lots of zeros and
         * then, the next time, lots of ones.  We decrement
         * the "negative" patterns and increment the "positive"
         * patterns to preserve this feature.
         */
        if(pattern & 0x80000000) {
                pattern = -pattern;	/* complement & increment */
        }
        else {
                pattern = ~pattern;
        }
        if (error_limit <= 0)
        {
            printf("Too many errors, printing stopped\n");
        }
        incr = -incr;
    }

}
#endif
/*
 * Breath some life into the board...
 *
 * The first part of initialization is running from Flash memory;
 * its main purpose is to initialize the RAM so that we
 * can relocate the monitor code to RAM.
 */

/*
 * All attempts to come up with a "common" initialization sequence
 * that works for all boards and architectures failed: some of the
 * requirements are just _too_ different. To get rid of the resulting
 * mess of board dependend #ifdef'ed code we now make the whole
 * initialization sequence configurable to the user.
 *
 * The requirements for any new initalization function is simple: it
 * receives a pointer to the "global data" structure as it's only
 * argument, and returns an integer return code, where 0 means
 * "continue" and != 0 means "fatal error, hang the system".
 */

int early_board_init(void);
int octeon_boot_bus_init_board(void);
#if CFG_LATE_BOARD_INIT
int late_board_init(void);
#endif

typedef int (init_fnc_t) (void);

init_fnc_t *init_sequence[] = {
#if CONFIG_OCTEON && (!CONFIG_OCTEON_SIM)
#if CONFIG_BOOT_BUS_BOARD
    octeon_boot_bus_init_board,  // board specific boot bus init
#else
    octeon_boot_bus_init,  // need to map PAL to read clocks, speed up flash
#endif
#endif
	timer_init,
	env_init,		/* initialize environment */
#if CONFIG_OCTEON
    early_board_init,
#endif
	init_baudrate,		/* initialze baudrate settings */
	serial_init,		/* serial communications setup */
	console_init_f,
	display_banner,		/* say that we are here */
#if !CONFIG_OCTEON_SIM
        init_dram,
#endif
#if DO_SIMPLE_DRAM_TEST_FROM_FLASH
        dram_test,
#endif
	checkboard,
	init_func_ram,
	NULL,
};
void check_manufacturing_data(gd_t *gd)
{
	uchar *flash_start, *ram_start;
	ManuInfo_t *manuinfo;
	char	cmd[100];

	flash_start = (uchar *)MANUFACTURING_FLASH_START; 
	ram_start = (uchar *)MANUFACTURING_RAM_START; 
	manuinfo = (ManuInfo_t *)MANUFACTURING_RAM_START; 

	/*if flash is empty, call default*/
	if((flash_start[0] == 0xFF) && (flash_start[1] == 0xFF)) {
		memset(manuinfo, 0x0, sizeof(manuinfo));
		switch (gd->board_desc.board_type) {
			case CVMX_BOARD_TYPE_CUST_SRX5308:
				sprintf((char *) manuinfo->ProductID, "SRX5308");
				break;
				break;
			default:
				sprintf((char *) manuinfo->ProductID, "Unknown");
				break;
		}
		manuinfo->ProductID[31] = 0x0;
		sprintf(manuinfo->HWVer, "00:00:A0:01");
		manuinfo->reginfo = 1;				//Regional information 0 WW, 1 NA, 2
		manuinfo->numofimages = 2;			//Number of images for the product
		manuinfo->currimage = 0x2A;			//Questionable field ? Pointer to current active image
		manuinfo->kernelchecksum[0]=0x0;
		manuinfo->kernelchecksum[1]=0x0;
		manuinfo->kernelsize[0]=0x0;
		manuinfo->kernelsize[1]=0x0;
               memcpy(manuinfo->version[0],'\0',16);
		memcpy(manuinfo->version[1],'\0',16);
		manuinfo->version[0][0]='0';
		manuinfo->version[1][0]='0';
		manuinfo->image_status[0]=0;
		manuinfo->image_status[1]=0;
		sprintf(manuinfo->basemac, "000000000001");
		memset(manuinfo->maccnt, 0x0, 4);	
		manuinfo->maccnt[0] = 21;
		sprintf(manuinfo->serno, "0000000000001");
		manuinfo->serno[31] = 0x0;
		manuinfo->boardinfo.cpuinfo = 0x5010;
		manuinfo->boardinfo.freq = gd->cpu_clock_mhz;
		manuinfo->boardinfo.ram = (gd->ram_size/(1024*1024));
		manuinfo->boardinfo.flash = (gd->bd->bi_flashsize/(1024*1024));
		manuinfo->boardinfo.poe_pwr = 0x0;
		manuinfo->boardinfo.portw1 = 0x7;		
		manuinfo->boardinfo.portw2 = 0x7;
		manuinfo->boardinfo.portw3 = 0x7;
		manuinfo->boardinfo.portw4 = 0x7;
		memset(manuinfo->boardinfo.portlan, 0x0, 8);	
		manuinfo->boardinfo.portlan[0] = 0xf;
		manuinfo->boardinfo.portlan[1] = 0xf;
		manuinfo->boardinfo.portlan[2] = 0xf;
		manuinfo->boardinfo.portlan[3] = 0xf;
		manuinfo->boardinfo.minipci[0] = 0x0;
		manuinfo->boardinfo.minipci[1] = 0x0;
		manuinfo->boardinfo.serialport = 0x1;

		memset(cmd, 0x0, 100);
		sprintf(cmd, "protect off 0x%x 0x%x", MANUFACTURING_FLASH_START, MANUFACTURING_FLASH_START+CFG_MD_SIZE-1);
		run_command(cmd, 0);

		memset(cmd, 0x0, 100);
		sprintf(cmd, "erase 0x%x 0x%x", MANUFACTURING_FLASH_START, MANUFACTURING_FLASH_START+CFG_MD_SIZE-1);
		run_command(cmd, 0);

		memset(cmd, 0x0, 100);
		sprintf(cmd, "cp.b 0x%x 0x%x 0x%x", MANUFACTURING_RAM_START, MANUFACTURING_FLASH_START,CFG_MD_SIZE);
		run_command(cmd, 0);

	}
	else {
		memcpy(ram_start, flash_start, sizeof(ManuInfo_t));
	}
}


void board_init_f(ulong bootflag)
{
	DECLARE_GLOBAL_DATA_PTR;

	gd_t gd_data, *id;
	bd_t *bd;
	init_fnc_t **init_fnc_ptr;
	ulong addr, addr_sp, len = (ulong)&uboot_end - CFG_MONITOR_BASE;
	uint32_t u_boot_mem_top; /* top of memory, as used by u-boot */
#ifdef CONFIG_PURPLE
	void copy_code (ulong);
#endif

        SET_K0(&gd_data);
        /* Pointer is writable since we allocated a register for it.  */
	memset ((void *)gd, 0, sizeof (gd_t));

        /* Enable soft BIST so that BIST is run on soft resets. */
        if (!OCTEON_IS_MODEL(OCTEON_CN38XX_PASS2) && !OCTEON_IS_MODEL(OCTEON_CN31XX))
            cvmx_write_csr(CVMX_CIU_SOFT_BIST, 1);

        /* Change default setting for CN58XX pass 2 */
        if (OCTEON_IS_MODEL(OCTEON_CN58XX_PASS2))
            cvmx_write_csr(CVMX_MIO_FUS_EMA, 2);

#ifdef ENABLE_BOARD_DEBUG
        int gpio_status_data = 1;
#endif
	for (init_fnc_ptr = init_sequence; *init_fnc_ptr; ++init_fnc_ptr) {
#ifdef ENABLE_BOARD_DEBUG
                gpio_status(gpio_status_data++);
#endif
		if ((*init_fnc_ptr)() != 0) {
			printf("hanging...\n");
			hang ();
		}
	}
	/*
	 * Now that we have DRAM mapped and working, we can
	 * relocate the code and continue running from DRAM.
	 */
#if defined(CONFIG_NO_RELOCATION) && defined(CONFIG_RAM_RESIDENT)
        /* If loaded into ram, we can skip relocation , and run from the spot we were loaded into */
	addr = CFG_MONITOR_BASE;
	u_boot_mem_top = CFG_SDRAM_BASE + MIN(gd->ram_size, (1*1024*1024));
#else
    /* Locate at top of first Megabyte */
	addr = CFG_SDRAM_BASE + MIN(gd->ram_size, (1*1024*1024));
	u_boot_mem_top = addr;


		/* We can reserve some RAM "on top" here.
		 */

		/* round down to next 4 kB limit.
		 */
	addr &= ~(4096 - 1);
#ifdef DEBUG
	printf ("Top of RAM usable for U-Boot at: %08lx\n", addr);
#endif


		/* Reserve memory for U-Boot code, data & bss
		 */
	addr -= MAX(len, (512*1024));
    /* round down to next 64k (allow us to create image at same addr for debugging)*/
	addr &= ~(64 * 1024 - 1);

#ifdef DEBUG
	printf ("Reserving %ldk for U-Boot at: %08lx\n", MAX(len, (512*1024)) >> 10, addr);
#endif

#endif  // CONFIG_RAM_RESIDENT


	 	/* Reserve memory for malloc() arena.
		 */
	addr_sp = addr - TOTAL_MALLOC_LEN;
#ifdef DEBUG
	printf ("Reserving %dk for malloc() at: %08lx\n",
			TOTAL_MALLOC_LEN >> 10, addr_sp);
#endif

	/*
	 * (permanently) allocate a Board Info struct
	 * and a permanent copy of the "global" data
	 */
	addr_sp -= sizeof(bd_t);
	bd = (bd_t *)addr_sp;
	gd->bd = bd;
#ifdef DEBUG
	printf ("Reserving %d Bytes for Board Info at: %08lx\n",
			sizeof(bd_t), addr_sp);
#endif
	addr_sp -= sizeof(gd_t);
	id = (gd_t *)addr_sp;
#ifdef DEBUG
	printf ("Reserving %d Bytes for Global Data at: %08lx\n",
			sizeof (gd_t), addr_sp);
#endif

	 	/* Reserve memory for boot params.
		 */
	addr_sp -= CFG_BOOTPARAMS_LEN;
	bd->bi_boot_params = addr_sp;
#ifdef DEBUG
	printf ("Reserving %dk for boot params at: %08lx\n",
			CFG_BOOTPARAMS_LEN >> 10, addr_sp);
#endif

	/*
	 * Finally, we set up a new (bigger) stack.
	 *
	 * Leave some safety gap for SP, force alignment on 16 byte boundary
	 * Clear initial stack frame
	 */
	addr_sp -= 16;
	addr_sp &= ~0xF;
    /* Changed to avoid lvalue cast warnings */
    uint32_t *tmp_ptr = (void *)addr_sp;
    *tmp_ptr-- = 0;
    *tmp_ptr-- = 0;
    addr_sp = (uint32_t)addr_sp;

#define STACK_SIZE  (16*1024UL)
    bd->bi_uboot_ram_addr = (addr_sp - STACK_SIZE) & ~(STACK_SIZE - 1);
    bd->bi_uboot_ram_used_size = u_boot_mem_top - bd->bi_uboot_ram_addr;

#ifdef DEBUG
	printf ("Stack Pointer at: %08lx, stack size: 0x%08x\n", addr_sp, STACK_SIZE);
    printf("Base DRAM address used by u-boot: 0x%08x, size: 0x%x\n", bd->bi_uboot_ram_addr, bd->bi_uboot_ram_used_size);
#endif
	/*
	 * Save local variables to board info struct
	 */
	bd->bi_memstart	= CFG_SDRAM_BASE;	/* start of  DRAM memory */
	bd->bi_memsize	= gd->ram_size;		/* size  of  DRAM memory in bytes */
	bd->bi_baudrate	= gd->baudrate;		/* Console Baudrate */

	memcpy (id, (void *)gd, sizeof (gd_t));

	/* On the purple board we copy the code in a special way
	 * in order to solve flash problems
	 */
#ifdef CONFIG_PURPLE
	copy_code(addr);
#endif
#ifdef DEBUG
        printf("relocating and jumping to code in DRAM\n");
#endif
	relocate_code (addr_sp, id, addr);

	/* NOTREACHED - relocate_code() does not return */
}
/************************************************************************
 *
 * This is the next part if the initialization sequence: we are now
 * running from RAM and have a "normal" C environment, i. e. global
 * data can be written, BSS has been cleared, the stack size in not
 * that critical any more, etc.
 *
 ************************************************************************
 */

void board_init_r (gd_t *id, ulong dest_addr)
{
    DECLARE_GLOBAL_DATA_PTR;
    cmd_tbl_t *cmdtp;
    ulong size = 0;
    extern void malloc_bin_reloc (void);
#ifndef CFG_ENV_IS_NOWHERE
    extern char * env_name_spec;
#endif
    char *s, *e;
    bd_t *bd;
    int i;

/* Convince GCC to really do what we want, otherwise the old value is still used......*/
    SET_K0(id);

    gd->flags |= GD_FLG_RELOC;  /* tell others: relocation done */

#ifdef DEBUG
    /* This is the address that u-boot has relocated itself to.  Use this 
    ** address for RELOCATED_TEXT_BASE in the makefile to enable the relocated
    ** build for debug info. */
    printf ("Now running in RAM - U-Boot at: 0x%x\n", dest_addr);
#endif

    gd->reloc_off = dest_addr - CFG_MONITOR_BASE;

    monitor_flash_len = (ulong)&uboot_end_data - dest_addr;

    /*
     * We have to relocate the command table manually
     */
    if (gd->reloc_off)
    {
        for (cmdtp = &__u_boot_cmd_start; cmdtp !=  &__u_boot_cmd_end; cmdtp++)
        {
            ulong addr;

            addr = (ulong) (cmdtp->cmd) + gd->reloc_off;
            cmdtp->cmd =
            (int (*)(struct cmd_tbl_s *, int, int, char *[]))addr;

            addr = (ulong)(cmdtp->name) + gd->reloc_off;
            cmdtp->name = (char *)addr;

            if (cmdtp->usage)
            {
                addr = (ulong)(cmdtp->usage) + gd->reloc_off;
                cmdtp->usage = (char *)addr;
            }
#ifdef	CFG_LONGHELP
            if (cmdtp->help)
            {
                addr = (ulong)(cmdtp->help) + gd->reloc_off;
                cmdtp->help = (char *)addr;
            }
#endif
        }
        /* there are some other pointer constants we must deal with */
#ifndef CFG_ENV_IS_NOWHERE
        env_name_spec += gd->reloc_off;
#endif

    }  /* if (gd-reloc_off) */


    size = flash_init();
    display_flash_config (size);

    bd = gd->bd;
    bd->bi_flashstart = CFG_FLASH_BASE;
    bd->bi_flashsize = size;
#if CFG_MONITOR_BASE == CFG_FLASH_BASE
    bd->bi_flashoffset = monitor_flash_len; /* reserved area for U-Boot */
#else
    bd->bi_flashoffset = 0;
#endif
     /* set default value in Product information */
     check_manufacturing_data(gd);

	 
    /* initialize malloc() area */
    mem_malloc_init();
    malloc_bin_reloc();

    /* relocate environment function pointers etc. */
    env_relocate();

#if !CONFIG_OCTEON_SIM
    if (octeon_is_model(OCTEON_CN38XX))
    {
        extern int octeon_ipd_bp_enable(void);
        extern int octeon_ipd_bp_verify(void);
        int loops = octeon_ipd_bp_enable();
        if (loops < 40)
        {
            if (octeon_ipd_bp_verify())
                printf("IPD backpressure workaround verified, took %d loops\n", loops);
            else
                printf("IPD backpressure workaround FAILED verfication. Adjust octeon_ipd_bp_enable() for this chip!\n");
        }
        else
            printf("IPD backpressure workaround FAILED. Adjust octeon_ipd_bp_enable() for this chip!\n");
    }


    /* Disable the watchdog timers, as these are still running after a soft reset as a PCI target */
    cvmx_write_csr(CVMX_CIU_WDOGX(0), 0);
    cvmx_write_csr(CVMX_CIU_WDOGX(1), 0);
    if (octeon_is_model(OCTEON_CN38XX) || octeon_is_model(OCTEON_CN58XX))
    {
        for (i = 2; i < 16;i++)
            cvmx_write_csr(CVMX_CIU_WDOGX(i), 0);
    }

    /* Flush and unlock L2 cache (soft reset does not unlock locked lines) */
    octeon_flush_l2_cache();
#endif

#if !CONFIG_OCTEON_SIM && !defined(CONFIG_NO_RELOCATION) && !defined(CONFIG_NO_CLEAR_DDR)
    /* Zero all of memory above DRAM_LATE_ZERO_OFFSET */
    printf("Clearing DRAM.....");
    uint64_t start_addr = DRAM_LATE_ZERO_OFFSET | (1ull << 63ull);
    octeon_bzero64_pfs(start_addr, (uint64_t)(MIN(256*1024*1024, gd->ram_size) - DRAM_LATE_ZERO_OFFSET));
    puts(".");

    if (gd->ram_size > 256*1024*1024)
    {
        octeon_bzero64_pfs(0x8000000410000000ull, (uint64_t)MIN((gd->ram_size - 256*1024*1024), 256*1024*1024));
        puts(".");
    }


    if (gd->ram_size > 512*1024*1024)
    {
        octeon_bzero64_pfs(0x8000000020000000ull, gd->ram_size - 512*1024*1024ull);
        puts(".");
    }
    printf(" done\n");
#endif

    /* Clear DDR/L2C ECC error bits */
    cvmx_write_csr(CVMX_L2D_ERR, cvmx_read_csr(CVMX_L2D_ERR));
    cvmx_write_csr(CVMX_L2T_ERR, cvmx_read_csr(CVMX_L2T_ERR));
    if (octeon_is_model(OCTEON_CN56XX)) {
        cvmx_l2c_cfg_t l2c_cfg;
        l2c_cfg.u64 = cvmx_read_csr(CVMX_L2C_CFG);
        if (l2c_cfg.cn56xx.dpres0) {
            cvmx_write_csr(CVMX_LMCX_MEM_CFG0(0), cvmx_read_csr(CVMX_LMCX_MEM_CFG0(0)));
        }
        if (l2c_cfg.cn56xx.dpres1) {
            cvmx_write_csr(CVMX_LMCX_MEM_CFG0(1), cvmx_read_csr(CVMX_LMCX_MEM_CFG0(1)));
        }
    } else {
        cvmx_write_csr(CVMX_LMC_MEM_CFG0, cvmx_read_csr(CVMX_LMC_MEM_CFG0));
    }

    if (!cvmx_bootmem_phy_mem_list_init(gd->ram_size, OCTEON_RESERVED_LOW_MEM_SIZE, malloc(sizeof(cvmx_bootmem_desc_t))))
    {
        printf("FATAL: Error initializing free memory list\n");
        while (1);
    }

    /* setup cvmx_sysinfo structure so we can use simple executive functions */
    octeon_setup_cvmx_sysinfo();

#ifdef DEBUG
    cvmx_bootmem_phy_list_print();
#endif

#if !CONFIG_OCTEON_SIM || defined(CONFIG_NET_MULTI) || defined(OCTEON_RGMII_ENET) || defined(OCTEON_MGMT_ENET) 
    {
        char *eptr;
        uint32_t addr;
        uint32_t size;

        eptr = getenv("octeon_reserved_mem_load_size");

        if (!eptr || !strcmp("auto", eptr))
        {
            /* Pick a size that we think is appropriate.
            ** Please note that for small memory boards this guess will likely not be ideal.
            ** Please pick a specific size for boards/applications that require it. */
   
            //size = MIN(448*1024*1024, (gd->ram_size/3) & ~0xFFFFF);
            /* add by mark_wens */
            	size=185*1024*1024;  /*using space of dr0  */
        }
        else
        {
            size = simple_strtol(eptr,NULL, 16);
        }
        if (size)
        {
            eptr = getenv("octeon_reserved_mem_load_base");
            if (!eptr || !strcmp("auto", eptr))
            {
                uint64_t mem_top;
                /* Leave some room for previous allocations that are made starting at the top of the low
                ** 256 Mbytes of DRAM */
                int adjust = 1024*1024;
                /* Put block at the top of DDR0, or bottom of DDR2 */
                if (gd->ram_size <= 512*1024*1024 || (size > gd->ram_size - 512*1024*1024))
                    mem_top = MIN(gd->ram_size - adjust, 256*1024*1024 - adjust);
                else
                {
                    /* We have enough room, so set mem_top so that block is at
                    ** base of  DDR2 segment */
                    mem_top = 512*1024*1024 + size;
                }
                addr = mem_top - size;
            }
            else
            {
                addr = simple_strtol(eptr,NULL, 16);
            }

            if (0 > cvmx_bootmem_phy_named_block_alloc(size, addr, addr + size, 0, OCTEON_BOOTLOADER_LOAD_MEM_NAME, 0))
            {
                printf("ERROR: Unable to allocate bootloader reserved memory (addr: 0x%x, size: 0x%x).\n", addr, size);
            }
            else
            {
                /* Set default load address to base of memory reserved for loading.
                ** The setting of the env. variable also sets the load_addr global variable.
                ** This environment variable is overridden each boot if a reserved block is created. */
                char str[20];
                sprintf(str, "0x%x", addr);
                setenv("loadaddr", str);
                load_reserved_addr = addr;
                load_reserved_size = size;
            }
        }
        else
            printf("WARNING: No reserved memory for image loading.\n");

    }
#endif

#if !CONFIG_OCTEON_SIM
    /* Reserve memory for Linux kernel.  The kernel requires specific physical addresses,
    ** so by reserving this here we can load simple exec applications and do other memory
    ** allocation without interfering with loading a kernel image.  This is freed and used
    ** when loading a Linux image.  If a Linux image is not loaded, then this is freed just
    ** before the applications start.
    */
    {
        char *eptr;
        uint32_t addr;
        uint32_t size;
        eptr = getenv("octeon_reserved_mem_linux_size");
        if (!eptr ||!strcmp("auto", eptr))
        {
            //size = (MIN(gd->ram_size, 128*1024*1024))/2;  // because we set 448mb
            size=64*1024*1024;
        }
        else
        {
            size = simple_strtol(eptr,NULL, 16);
        }
        if (size)
        {
            eptr = getenv("octeon_reserved_mem_linux_base");
            if (!eptr ||!strcmp("auto", eptr))
            {
                /* Start right after the bootloader */
                addr = OCTEON_RESERVED_LOW_MEM_SIZE;
            }
            else
            {
                addr = simple_strtol(eptr,NULL, 16);
            }
            /* Allocate the memory, and print message if we fail */
            if (0 > cvmx_bootmem_phy_named_block_alloc(size, addr, addr + size, 0, OCTEON_LINUX_RESERVED_MEM_NAME, 0))
                printf("ERROR: Unable to allocate linux reserved memory (addr: 0x%x, size: 0x%x).\n", addr, size);
        }

    }
#endif


    /* Put bootmem descriptor address in known location for host */
     {
         /* Make sure it is not in kseg0, as we want physical addresss */
         uint64_t *u64_ptr = (void *)(0x80000000 | BOOTLOADER_BOOTMEM_DESC_ADDR);
         *u64_ptr = ((uint32_t)__cvmx_bootmem_internal_get_desc_ptr()) & 0x7fffffffull;
     }

#ifdef CFG_PCI_CONSOLE
     /* Check to see if we need to set up a PCI console block. Set up structure if 
     ** either count is set (and greater than 0) or pci_console_active is set */
     char *ep = getenv("pci_console_count");
     if ((ep && (simple_strtoul(ep, NULL, 10) > 0)) || getenv("pci_console_active"))
     {
         /* Initialize PCI console data structures */
         int console_count;
         int console_size;
         ep = getenv("pci_console_count");
         if (ep)
             console_count = simple_strtoul(ep, NULL, 10);
         else
             console_count = 1;

         if (console_count < 1)
             console_count = 1;

         ep = getenv("pci_console_size");
         if (ep)
             console_size = simple_strtoul(ep, NULL, 10);
         else
             console_size = 1024;

         if (console_size < 128)
             console_size = 128;

         pci_cons_desc_addr = octeon_pci_console_init(console_count, console_size);
         if (!pci_cons_desc_addr)
         {
             printf("WARNING: PCI console init FAILED\n");
         }
         else
         {
             printf("PCI console init succeeded, %d consoles, %d bytes each\n", console_count, console_size);
         }
     }
#endif



#ifdef DEBUG
     cvmx_bootmem_phy_list_print();
#endif


#if (CONFIG_OCTEON_EEPROM_TUPLES)
    {
        /* Read coremask from eeprom, and set environment variable if present.
        ** Print warning if not present in EEPROM.
        ** Currently ignores voltage/freq fields, and just uses first capability tuple
        ** This is only valid for certain parts.
        */
        int addr;
        uint8_t ee_buf[OCTEON_EEPROM_MAX_TUPLE_LENGTH];
        octeon_eeprom_chip_capability_t *cc_ptr = (void *)ee_buf;
        addr = octeon_tlv_get_tuple_addr(CFG_DEF_EEPROM_ADDR, EEPROM_CHIP_CAPABILITY_TYPE, 0, ee_buf, OCTEON_EEPROM_MAX_TUPLE_LENGTH);
        if (addr >= 0 && 
            (((octeon_is_model(OCTEON_CN38XX_PASS1) || (octeon_is_model(OCTEON_CN38XX) && (cvmx_read_csr(CVMX_CIU_FUSE) & 0xffff) == 0xffff && !cvmx_octeon_fuse_locked())))
            || octeon_is_model(OCTEON_CN56XX) || octeon_is_model(OCTEON_CN52XX)
            ))
        {
            char tmp[10];
            sprintf(tmp,"0x%04x", cc_ptr->coremask);
            setenv("coremask_override", tmp);
            coremask_from_eeprom = cc_ptr->coremask;  /* Save copy for later verification */
        }
        else
        {
            /* No chip capability tuple found, so we need to check if we expect one.
            ** CN31XX chips will all have fuses blown appropriately.
            ** CN38XX chips may have fuses blown, but may not.  We will check to see if
            ** we need a chip capability tuple and only warn if we need it but don't have it.
            */

            if (octeon_is_model(OCTEON_CN38XX_PASS1))
            {
                /* always need tuple for pass 1 chips */
                printf("Warning: No chip capability tuple found in eeprom, coremask_override may be set incorrectly\n");
            }
            else if (octeon_is_model(OCTEON_CN38XX))
            {
                /* Here we only need it if no core fuses are blown and the lockdown fuse is not blown.
                ** In all other cases the cores fuses are definitive and we don't need a coremask override
                */
                if ((cvmx_read_csr(CVMX_CIU_FUSE) & 0xffff) == 0xffff && !cvmx_octeon_fuse_locked())
                {
                    printf("Warning: No chip capability tuple found in eeprom, coremask_override may be set incorrectly\n");
                }
                else
                {
                    /* Clear coremask_override as we have a properly fused part, and don't need it */
                    setenv("coremask_override", NULL);
                }
            }
            else
            {
                /* 31xx and 30xx will always have core fuses blown appropriately */

                setenv("coremask_override", NULL);
            }
        }
    }
#endif


    /* Set numcores env variable to indicate the number of cores available */
    char tmp[10];
    sprintf(tmp,"%d", octeon_get_available_core_count());
    setenv("numcores", tmp);

#if (!CONFIG_OCTEON_SIM)
    octeon_bist();
#endif


    /* board MAC address */
    s = getenv ("ethaddr");
    for (i = 0; i < 6; ++i)
    {
        bd->bi_enetaddr[i] = s ? simple_strtoul (s, &e, 16) : 0;
        if (s)
            s = (*e) ? e + 1 : e;
    }

    /* IP Address */
    bd->bi_ip_addr = getenv_IPaddr("ipaddr");

#if defined(CONFIG_PCI)
    /*
     * Do pci configuration
     */
    pci_init();
#endif

/** leave this here (after malloc(), environment and PCI are working) **/
    /* Initialize devices */
    devices_init ();

    /* Must change std* variables after devices_init()
    ** Force bootup settings based on pci_console_active, overriding the std* variables */
    if (getenv("pci_console_active"))
    {
        printf("Using PCI console, serial port disabled.\n");
        setenv("stdin", "pci");
        setenv("stdout", "pci");
        setenv("stderr", "pci");
        octeon_led_str_write("PCI CONS");
    }
    else
    {
        setenv("stdin", "serial");
        setenv("stdout", "serial");
        setenv("stderr", "serial");
    }


    jumptable_init ();

    /* Initialize the console (after the relocation and devices init) */
    console_init_r ();
/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/

    if (getenv("pci_console_active"))
    {
        /* We want to print out the board/chip info again, as this is very useful for automated testing.
        ** and board identification. */
        int cpu_rev = cvmx_get_proc_id() & 0xff;
        printf("%s board revision major:%d, minor:%d, serial #: %s\n",
               cvmx_board_type_to_string(gd->board_desc.board_type),
               gd->board_desc.rev_major, gd->board_desc.rev_minor,
               gd->board_desc.serial_str);

        printf("OCTEON %s-%s revision: %d, Core clock: %d MHz, DDR clock: %d MHz (%d Mhz data rate)\n",
               octeon_get_model_name(), octeon_get_submodel_name(),
               cpu_rev & ~0x10,
               gd->cpu_clock_mhz,
               gd->ddr_clock_mhz, gd->ddr_clock_mhz *2);
    }

    /* Initialize from environment. Note that in most cases the environment
    ** variable will have been set from the reserved memory block allocated above.
    ** By also checking this here, this allows the load address to be set in cases
    ** where there is no reserved block configured. 
    ** If loadaddr is not set, set it to something.  This default should never really be used,
    ** but we need to handle this case. */
    if ((s = getenv ("loadaddr")) != NULL)
    {
        load_addr = simple_strtoul (s, NULL, 16);
    }
    else
    {
        uint32_t addr = 0x100000;
        char tmp[20];
#ifdef CFG_LOAD_ADDR
        addr = CFG_LOAD_ADDR;
#endif
        sprintf(tmp, "0x%x", addr);
#if !CONFIG_OCTEON_SIM
        printf("WARNING: loadaddr not set, defaulting to %s.  Please either define a load reserved block or set the loadaddr environment variable.\n", tmp);
#endif
        setenv("loadaddr", tmp);
    }


#if !CONFIG_OCTEON_SIM
    {
        char tmp[30];
        /* Set up a variety of environment variables for use by u-boot and
        ** applications (through environment named block). */

        /* Set env_addr and env_size variables to the address and size of the u-boot environment. */
        sprintf(tmp,"0x%x", CFG_ENV_ADDR);
        setenv("env_addr", tmp);
        sprintf(tmp,"0x%x", CFG_ENV_SIZE);
        setenv("env_size", tmp);


        /* Describe physical flash */
        sprintf(tmp,"0x%x", CFG_FLASH_BASE);
        setenv("flash_base_addr", tmp);
        sprintf(tmp,"0x%x", CFG_FLASH_SIZE);
        setenv("flash_size", tmp);

#ifdef CFG_NORMAL_BOOTLOADER_SIZE
        /* We always want to report the address/size of the normal bootloader,
        ** so that we can use these from the failsafe bootloader to update the normal
        ** bootloader.  Updating the failsafe bootloader is always a 'manual' process. */
        sprintf(tmp,"0x%x", CFG_FLASH_BASE + CFG_FAILSAFE_BOOTLOADER_SIZE);
        setenv("uboot_flash_addr", tmp);
        sprintf(tmp,"0x%x", CFG_NORMAL_BOOTLOADER_SIZE);
        setenv("uboot_flash_size", tmp);

        /* Set addr size for flash space that is available for application use */
        sprintf(tmp,"0x%x", CFG_FLASH_BASE + CFG_FAILSAFE_BOOTLOADER_SIZE + CFG_NORMAL_BOOTLOADER_SIZE);
        setenv("flash_unused_addr", tmp);
        sprintf(tmp,"0x%x", CFG_FLASH_SIZE - CFG_FAILSAFE_BOOTLOADER_SIZE - CFG_NORMAL_BOOTLOADER_SIZE - CFG_ENV_SIZE);
        setenv("flash_unused_size", tmp);
#endif

    }
#endif

#if (CONFIG_COMMANDS & CFG_CMD_NET)
    if ((s = getenv ("bootfile")) != NULL)
    {
        copy_filename (BootFile, s, sizeof (BootFile));
    }
#endif	/* CFG_CMD_NET */

#if defined(CONFIG_MISC_INIT_R)
    /* miscellaneous platform dependent initialisations */
    misc_init_r ();
#endif

#if CONFIG_OCTEON_LANBYPASS
    /* Set the LAN bypass defaults for Thunder */
    octeon_gpio_cfg_output(6); /* GPIO 6 controls the bypass enable */
    octeon_gpio_cfg_output(7); /* GPIO 7 controls the watchdog clear */
    octeon_gpio_cfg_input(5);  /* GPIO 5 tells you the bypass status */
    
    /* Startup with bypass disabled and watchdog cleared */
    octeon_gpio_clr(6);
    octeon_gpio_set(7);
#endif 

#if (CONFIG_COMMANDS & CFG_CMD_NET) && defined(CONFIG_NET_MULTI)
    puts ("Net:   ");
    eth_initialize(gd->bd);
#endif


#if CONFIG_OCTEON && !CONFIG_OCTEON_SIM && (CONFIG_COMMANDS & CFG_CMD_IDE)
#ifndef CONFIG_PCI  
    if (1 || octeon_cf_present())
#endif
    {
        /* Scan for IDE controllers and register them in the ide_dev_desc array. 
        ** The order of scanning will determine what order they are listed to the user in. */

        /* Scan the DMA capable slot first, so that it is contoller 0 on the ebh5600 board. (and will likely
        ** be the only one populated on later board builds */
#if defined(OCTEON_CF_TRUE_IDE_CS0_ADDR)
        /* Init IDE structures for CF interface on boot bus */
        ide_register_device(IF_TYPE_IDE, BLOCK_QUIRK_IDE_CF | BLOCK_QUIRK_IDE_CF_TRUE_IDE, 0, MAKE_XKPHYS(OCTEON_CF_TRUE_IDE_CS0_ADDR), MAKE_XKPHYS(OCTEON_CF_TRUE_IDE_CS0_ADDR), 0, 0);
        if (CFG_IDE_MAXDEVICE > 1)
        {
            /* This device won't every be used, but we want to have 2 devices per ide bus regargless.  If we
            ** are only supporting 1 device, skip this */
            ide_register_device(IF_TYPE_IDE, BLOCK_QUIRK_IDE_CF | BLOCK_QUIRK_IDE_CF_TRUE_IDE, 1, MAKE_XKPHYS(OCTEON_CF_TRUE_IDE_CS0_ADDR), MAKE_XKPHYS(OCTEON_CF_TRUE_IDE_CS0_ADDR), 0, 0);
        }
#endif

#ifdef OCTEON_CF_COMMON_BASE_ADDR
        /* Init IDE structures for CF interface on boot bus */
        ide_register_device(IF_TYPE_IDE, BLOCK_QUIRK_IDE_CF, 0, MAKE_XKPHYS(OCTEON_CF_COMMON_BASE_ADDR + CFG_ATA_DATA_OFFSET), MAKE_XKPHYS(OCTEON_CF_COMMON_BASE_ADDR + CFG_ATA_REG_OFFSET), 0, 0);
        if (CFG_IDE_MAXDEVICE > 1)
        {
            /* This device won't every be used, but we want to have 2 devices per ide bus regargless.  If we
            ** are only supporting 1 device, skip this */
            ide_register_device(IF_TYPE_IDE, BLOCK_QUIRK_IDE_CF, 1, MAKE_XKPHYS(OCTEON_CF_COMMON_BASE_ADDR + CFG_ATA_DATA_OFFSET), MAKE_XKPHYS(OCTEON_CF_COMMON_BASE_ADDR + CFG_ATA_REG_OFFSET), 0, 0);
        }
#endif

        /* Always do ide_init if PCI enabled to allow for PCI IDE devices */
        ide_init();
    }
#endif

#if CONFIG_OCTEON
    /* verify that boot_init_vector type is the correct size */
    if (BOOT_VECTOR_NUM_WORDS*4 != sizeof(boot_init_vector_t))
    {
        printf("ERROR: boot_init_vector_t size mismatch: define: %d, type: %d\n",BOOT_VECTOR_NUM_WORDS*8,  sizeof(boot_init_vector_t));
        while (1)
            ;
    }
    /* DEBUGGER setup  */

    extern void cvmx_debugger_handler2(void);
    extern void cvmx_debugger_handler_pci(void);
    extern void cvmx_debugger_handler_pci_profiler(void);
    /* put relocated address of 2nd stage of debug interrupt
    ** at BOOTLOADER_DEBUG_TRAMPOLINE (word value)
    ** This only needs to be done once during initial boot
    */
    {
        /* Put address of cvmx_debugger_handler2() into fixed DRAM
        ** location so that minimal exception handler in flash
        ** can jump to this easily. Also put a list of the available handlers
        ** in a location accessible from PCI at a fixed address. Host PCI
        ** routines can then use this to replace the normal debug handler
        ** with one supporting other stuff.
        */
        uint32_t *ptr = (void *)BOOTLOADER_DEBUG_TRAMPOLINE;
        *ptr = (uint32_t)cvmx_debugger_handler2;
        ptr = (void*)BOOTLOADER_DEBUG_HANDLER_LIST;
        *ptr++ = (uint32_t)cvmx_debugger_handler2;
        *ptr++ = (uint32_t)cvmx_debugger_handler_pci;
        *ptr++ = (uint32_t)cvmx_debugger_handler_pci_profiler;
        *ptr = 0;

        /* Save the GP value currently used (after relocation to DRAM)
        ** so that debug stub can restore it before calls cvmx_debugger_handler2.
        */
        uint32_t tmp = 0x80000000 | BOOTLOADER_DEBUG_GP_REG;
        asm volatile ("sw    $28, 0(%[tmp])" : :[tmp] "r" (tmp) :"memory");
        /* Save K0 (global data pointer) so that debug stub can set up the correct
        ** context to use bootloader functions. */
        tmp = 0x80000000 | BOOTLOADER_DEBUG_KO_REG_SAVE;
        asm volatile ("sw    $26, 0(%[tmp])" : :[tmp] "r" (tmp) :"memory");
       
    }
#endif


#if CFG_LATE_BOARD_INIT
    late_board_init();
#endif

    /* main_loop() can return to retry autoboot, if so just run it again. */
    for (;;)
    {
        main_loop ();
    }

    /* NOTREACHED - no way out of command loop except booting */
}

void hang (void)
{
	puts ("### ERROR ### Please RESET the board ###\n");
	for (;;);
}

#ifdef ENABLE_BOARD_DEBUG
void gpio_status(int data)
{
    int i;
    for (i=0; i<7; ++i) {
        if ((data>>i)&1)
            octeon_gpio_set(8+i);
        else
            octeon_gpio_clr(8+i);
    }
}
#endif
