/*
 * (C) Copyright 2001, 2002
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 * Keith Outwater, keith_outwater@mvis.com`
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Date & Time support (no alarms) for Dallas Semiconductor (now Maxim)
 * DS1337 Real Time Clock (RTC).
 */

#include <common.h>
#include <command.h>
#include <rtc.h>
#include <i2c.h>

#if defined(CONFIG_RTC_DS1337) && (CONFIG_COMMANDS & CFG_CMD_DATE)

/*---------------------------------------------------------------------*/
#undef DEBUG_RTC

#ifdef DEBUG_RTC
#define DEBUGR(fmt,args...) printf(fmt ,##args)
#else
#define DEBUGR(fmt,args...)
#endif
/*---------------------------------------------------------------------*/

//#ifndef CFG_I2C_RTC_ADDR
# define CFG_I2C_RTC_ADDR	0x68
//#endif


/*
 * RTC register addresses
 */
#define RTC_SEC_REG_ADDR	0x0
#define RTC_MIN_REG_ADDR	0x1
#define RTC_HR_REG_ADDR		0x2
#define RTC_DAY_REG_ADDR	0x3
#define RTC_DATE_REG_ADDR	0x4
#define RTC_MON_REG_ADDR	0x5
#define RTC_YR_REG_ADDR		0x6
#define RTC_CTL_REG_ADDR	0x0e
#define RTC_STAT_REG_ADDR	0x0f

/*
 * RTC control register bits
 */
#define RTC_CTL_BIT_A1IE	0x1	/* Alarm 1 interrupt enable     */
#define RTC_CTL_BIT_A2IE	0x2	/* Alarm 2 interrupt enable     */
#define RTC_CTL_BIT_INTCN	0x4	/* Interrupt control            */
#define RTC_CTL_BIT_RS1		0x8	/* Rate select 1                */
#define RTC_CTL_BIT_RS2		0x10	/* Rate select 2                */
#define RTC_CTL_BIT_DOSC	0x80	/* Disable Oscillator           */

/*
 * RTC status register bits
 */
#define RTC_STAT_BIT_A1F	0x1	/* Alarm 1 flag                 */
#define RTC_STAT_BIT_A2F	0x2	/* Alarm 2 flag                 */
#define RTC_STAT_BIT_OSF	0x80	/* Oscillator stop flag         */


static uchar rtc_read (uchar reg);
static void rtc_write (uchar reg, uchar val);
static uchar bin2bcd (unsigned int n);
static unsigned bcd2bin (uchar c);
void rtc_read_2 (uchar reg, uchar * val);
void rtc_write_2 (uchar reg, uchar * val, int nbyte);



/*
 * Get the current time from the RTC
 */
void rtc_dotest ()
{
	printf("ds1337 rtc_dotest\n");

//	uchar tmp_char[16];
//	rtc_read_2(0x00,tmp_char);

	uchar tmp;
 	tmp = rtc_read (RTC_CTL_REG_ADDR);
	printf("the Control reg = %2x\n",tmp);
	tmp = rtc_read (RTC_STAT_REG_ADDR);
	printf("the State reg = %2x\n",tmp);

	uchar tmp_char[2];
	tmp_char[0] = 0x1F;
	tmp_char[1] = 0x0;
	rtc_write_2 (RTC_CTL_REG_ADDR, tmp_char,2);

	

//	uchar sec, min, hour, mday, wday, mon_cent, year, control, status;
/*
	control = rtc_read (RTC_CTL_REG_ADDR);
	status = rtc_read (RTC_STAT_REG_ADDR);
	sec = rtc_read (RTC_SEC_REG_ADDR);
	min = rtc_read (RTC_MIN_REG_ADDR);
	hour = rtc_read (RTC_HR_REG_ADDR);
	wday = rtc_read (RTC_DAY_REG_ADDR);
	mday = rtc_read (RTC_DATE_REG_ADDR);
	mon_cent = rtc_read (RTC_MON_REG_ADDR);
	year = rtc_read (RTC_YR_REG_ADDR);
*/
/*
	control = tmp_char[14];
	status =  tmp_char[15];
	sec =  tmp_char[0];
	min = tmp_char[1];
	hour = tmp_char[2];
	wday = tmp_char[3];
	mday = tmp_char[4];
	mon_cent = tmp_char[5];
	year = tmp_char[6];
*/

}

/*
 * Get the current time from the RTC
 */
void rtc_dowrite (char* choice)
{
	printf("ds1337 rtc_dowrite\n");
	printf("choice = %s\n",choice);

//	uchar tmp_char[16];
//	rtc_read_2(0x00,tmp_char);
	uchar tmp;

	if ( strcmp(choice,"0")==0)
	{
		rtc_write (RTC_CTL_REG_ADDR, 0x1F );
		printf("Write RTC Control 0x1F");
	}
	else if (strcmp(choice,"1")==0)
	{
		rtc_write (RTC_STAT_REG_ADDR, 0x0);
		printf("Write RTC State 0x0");
	}
	else if ( strcmp(choice,"2")==0)
	{
		uchar tmp_char[2];
		tmp_char[0] = 0x1F;
		tmp_char[1] = 0x0;
		rtc_write_2 (RTC_CTL_REG_ADDR, tmp_char,2);
	}
	else if ( strcmp(choice,"3")==0)
	{
		uchar tmp_char16[16];
		rtc_read_2(0x00,tmp_char16);
		tmp_char16[7] = 0xbf;
		tmp_char16[8] = 0xc3;
		tmp_char16[9] = 0xff;
		tmp_char16[10] = 0xff;
		tmp_char16[11] = 0xa8;
		tmp_char16[12] = 0x7f;
		tmp_char16[13] = 0x18;
		tmp_char16[14] = 0x1F;
		tmp_char16[15] = 0x0;
		rtc_write_2 (RTC_SEC_REG_ADDR, tmp_char16,16);

	}


}

/*
 * Get the current time from the RTC
 */
void rtc_doread (char* choice)
{
	printf("ds1337 rtc_doread\n");

	uchar tmp_char[16];
	uchar sec, min, hour, mday, wday, mon_cent, year, control, status;
	uchar tmp;

	printf("choice = %s\n",choice);
	if ( strcmp(choice,"0")==0 )
	{
		tmp = rtc_read (RTC_CTL_REG_ADDR);
		printf("the Control reg = %2x\n",tmp);
	}
	else if ( strcmp(choice,"1")==0)
	{
		tmp = rtc_read (RTC_STAT_REG_ADDR);
		printf("the State reg = %2x\n",tmp);
	}
	else if  (strcmp(choice,"2")==0)
	{
		rtc_read_2(0x00,tmp_char);
		control = tmp_char[14];
		status =  tmp_char[15];
		sec =  tmp_char[0];
		min = tmp_char[1];
		hour = tmp_char[2];
		wday = tmp_char[3];
		mday = tmp_char[4];
		mon_cent = tmp_char[5];
		year = tmp_char[6];

		printf ("Get RTC year: %02x mon/cent: %02x mday: %02x wday: %02x \nhr: %02x min: %02x sec: %02x control: %02x status: %02x\n",
		year, mon_cent, mday, wday, hour, min, sec, control, status);
	}


}

/*
 * Get the current time from the RTC
 */
void rtc_get (struct rtc_time *tmp)
{
	uchar tmp_char[16];
	uchar sec, min, hour, mday, wday, mon_cent, year, control, status;
	rtc_read_2(0x00,tmp_char);



	control = tmp_char[14];
	status =  tmp_char[15];
	sec =  tmp_char[0];
	min = tmp_char[1];
	hour = tmp_char[2];
	wday = tmp_char[3];
	mday = tmp_char[4];
	mon_cent = tmp_char[5];
	year = tmp_char[6];


	DEBUGR ("Get RTC year: %02x mon/cent: %02x mday: %02x wday: %02x "
		"hr: %02x min: %02x sec: %02x control: %02x status: %02x\n",
		year, mon_cent, mday, wday, hour, min, sec, control, status);

	if (status & RTC_STAT_BIT_OSF) {
		printf ("### Warning: RTC oscillator has stopped\n");
		/* clear the OSF flag */
		rtc_write (RTC_STAT_REG_ADDR,
			   rtc_read (RTC_STAT_REG_ADDR) & ~RTC_STAT_BIT_OSF);
	}

	tmp->tm_sec  = bcd2bin (sec & 0x7F);
	tmp->tm_min  = bcd2bin (min & 0x7F);
	tmp->tm_hour = bcd2bin (hour & 0x3F);
	tmp->tm_mday = bcd2bin (mday & 0x3F);
	tmp->tm_mon  = bcd2bin (mon_cent & 0x1F);
	tmp->tm_year = bcd2bin (year) + ((mon_cent & 0x80) ? 2000 : 1900);
	tmp->tm_wday = bcd2bin ((wday - 1) & 0x07);
	tmp->tm_yday = 0;
	tmp->tm_isdst= 0;

	DEBUGR ("Get DATE: %4d-%02d-%02d (wday=%d)  TIME: %2d:%02d:%02d\n",
		tmp->tm_year, tmp->tm_mon, tmp->tm_mday, tmp->tm_wday,
		tmp->tm_hour, tmp->tm_min, tmp->tm_sec);


	//set control state to RTC 

//	uchar tmp_char[16];
/*
	tmp_char[14] = 0x18;
	tmp_char[15] = 0x0;
	rtc_write_2 (RTC_SEC_REG_ADDR, tmp_char,16);

*/
}


/*
 * Set the RTC
 */
void rtc_set (struct rtc_time *tmp)
{
	uchar century;

	DEBUGR ("Set DATE: %4d-%02d-%02d (wday=%d)  TIME: %2d:%02d:%02d\n",
		tmp->tm_year, tmp->tm_mon, tmp->tm_mday, tmp->tm_wday,
		tmp->tm_hour, tmp->tm_min, tmp->tm_sec);

/*
	rtc_write (RTC_YR_REG_ADDR, bin2bcd (tmp->tm_year % 100));

	century = (tmp->tm_year >= 2000) ? 0x80 : 0;
	rtc_write (RTC_MON_REG_ADDR, bin2bcd (tmp->tm_mon) | century);

	rtc_write (RTC_DAY_REG_ADDR, bin2bcd (tmp->tm_wday + 1));
	rtc_write (RTC_DATE_REG_ADDR, bin2bcd (tmp->tm_mday));
	rtc_write (RTC_HR_REG_ADDR, bin2bcd (tmp->tm_hour));
	rtc_write (RTC_MIN_REG_ADDR, bin2bcd (tmp->tm_min));
	rtc_write (RTC_SEC_REG_ADDR, bin2bcd (tmp->tm_sec));
*/

	uchar tmp_char[16];
	tmp_char[0] = bin2bcd (tmp->tm_sec) ;
	tmp_char[1] = bin2bcd (tmp->tm_min);
	tmp_char[2] = bin2bcd (tmp->tm_hour);
	tmp_char[3] = (tmp->tm_wday + 1);
	tmp_char[4] = bin2bcd (tmp->tm_mday);
	century = (tmp->tm_year >= 2000) ? 0x80 : 0;
	tmp_char[5] = bin2bcd (tmp->tm_mon) | century;
	tmp_char[6] = bin2bcd (tmp->tm_year % 100);
	/*
	tmp_char[7] = 0xbf;
	tmp_char[8] = 0xc3;
	tmp_char[9] = 0xff;
	tmp_char[10] = 0xff;
	tmp_char[11] = 0xa8;
	tmp_char[12] = 0x7f;
	tmp_char[13] = 0x18;
	tmp_char[14] = 0x1F;
	tmp_char[15] = 0x0;
	*/
	rtc_write_2 (RTC_SEC_REG_ADDR, tmp_char,7);



	DEBUGR ("Set Mon = %x;  Day = %x; Hour = %x; Min =%x; Sec = %x;\n",
	bin2bcd (tmp->tm_mon),bin2bcd (tmp->tm_wday + 1),bin2bcd (tmp->tm_hour),  bin2bcd (tmp->tm_min),
	bin2bcd (tmp->tm_sec));
}


/*
 * Reset the RTC.  We also enable the oscillator output on the
 * SQW/INTB* pin and program it for 32,768 Hz output. Note that
 * according to the datasheet, turning on the square wave output
 * increases the current drain on the backup battery from about
 * 600 nA to 2uA.
 */
void rtc_reset (void)
{
	rtc_write (RTC_CTL_REG_ADDR, RTC_CTL_BIT_RS1 | RTC_CTL_BIT_RS2);
}


/*
 * Helper functions
 */

static uchar rtc_read (uchar reg)
{
	//return (i2c_reg_read (CFG_I2C_RTC_ADDR, reg));


	char buf;
	int Ri2c;
	//Ri2c =  i2c_read(CFG_I2C_RTC_ADDR, reg, 1, &buf, 1);
	Ri2c =  i2c_read(CFG_I2C_RTC_ADDR, reg, 2, &buf, 1); // change alen = 2
 /**
 *	int  i2c_read(uchar chip, uint addr, int alen, uchar *buffer, int len)
 * Reads bytes from eeprom and copies to DRAM.
 * Only supports address size of 2 (16 bit internal address.)
 * 
 * @param chip   chip address
 * @param addr   internal address (only 16 bit addresses supported)
 * @param alen   address length, must be 2
 * @param buffer memory buffer pointer
 * @param len    number of bytes to read
 * 
 * @return 0 on Success
 *         1 on Failure
 *

*/
/*
	if ( Ri2c )
	{
		printf(" read i2c Fail!!\n");
	}
	else
	{
		printf(" read i2c OK!!\n");
	}

	printf("Address = %x,buf = %x\n", reg,buf );
	  return(buf);
*/
}

void rtc_read_2 (uchar reg, uchar * val)
{
	//return (i2c_reg_read (CFG_I2C_RTC_ADDR, reg));


	char buf;
	int Ri2c;
	//Ri2c =  i2c_read(CFG_I2C_RTC_ADDR, reg, 1, &buf, 1);
	Ri2c =  i2c_read(CFG_I2C_RTC_ADDR, reg, 2, val, 16); // change alen = 2

/*
	if ( Ri2c )
	{
		printf(" read i2c Fail!!\n");
	}
	else
	{
		printf(" read i2c OK!!\n");
	}

	int i;
	for (i=0;i<16;i++)
{
		printf("Address = %d,buf = %2x\n", i,*(val+i) );
	}
	 // return(buf);
*/
}


static void rtc_write (uchar reg, uchar val)
{

	//i2c_reg_write (CFG_I2C_RTC_ADDR, reg, val);
	int Ri2c;
	uchar buf;
	buf = val;
	
	Ri2c =  i2c_write(CFG_I2C_RTC_ADDR, reg, 2, &buf, 1);// modify by Joe
}

void rtc_write_2 (uchar reg, uchar * val, int nbyte)
{

	//i2c_reg_write (CFG_I2C_RTC_ADDR, reg, val);
	int Ri2c;
	//u16 buf;
	//buf = val;
	
	Ri2c =  i2c_write(CFG_I2C_RTC_ADDR, reg, 2, val, nbyte);// modify by Joe
}

static unsigned bcd2bin (uchar n)
{
	return ((((n >> 4) & 0x0F) * 10) + (n & 0x0F));
}

static unsigned char bin2bcd (unsigned int n)
{
	return (((n / 10) << 4) | (n % 10));
}

#endif /* CONFIG_RTC_DS1337 && (CFG_COMMANDS & CFG_CMD_DATE) */
